
##Do test function:
##=================

doTest <- function(x, type="wald", df.adjustment=NULL, hac=TRUE)
{
  ##initiate
  ##--------
  
  mIt <- x
  mIt <- dropvar(mIt) #any identical columns?
  iT <- NROW(mIt)
  S <- NCOL(mIt)
  HACvcov <- ifelse(hac==TRUE, TRUE, FALSE)
  DFadjust <- ifelse( !is.null(df.adjustment), TRUE, FALSE)
  doWaldTest <- ifelse(type=="wald", TRUE, FALSE)
  doLMtest <- ifelse(type=="lm", TRUE, FALSE)

  ##lag truncation
  ##--------------

  if( HACvcov==FALSE ){
    iL <- 0 #assume scores not autocorrelated
  }
  if( HACvcov==TRUE ){
    ##S_T function:
    iS_T <- 2*(iT/100)^(2/9)
    ##lag truncation:
    iS_T <- floor(iS_T)
    iL <- max(1,iS_T)   #note: j/iS_T > 1 when j > i
    ##Bartlett weights:
    vW <- 1 - 1:iL/iS_T #Hansen (1992)
  }

  ##compute DF-adjustment
  ##---------------------

  DFwaldAdj <- 1
  DFlmAdj <- 1
  if( DFadjust ){
    DFwaldAdj <- df.adjustment
    DFlmAdj <- df.adjustment
  }

  ##build restriction matrix
  ##------------------------

  ##version 1:
  tmp1 <- matrix(0, S-1, S-1)
  diag(tmp1) <- 1
  tmp2 <- matrix(0, S-1, S-1)
  diag(tmp2) <- -1
  tmp1 <- cbind(tmp1,0)
  tmp2 <- cbind(0,tmp2)
  mR <- tmp1 + tmp2

  ##transpose R:
  mRt <- t(mR)

  ##estimate pi1 = E(It)
  ##-------------------

  ##unconstrained estimator:
  pi1hat <- colMeans(mIt)
  pi1hat <- pmin(pi1hat, 1-.Machine$double.eps) #ensure no exact 1s
  pi1hat <- pmax(pi1hat, .Machine$double.eps) #ensure no exact 0s

  ##constrained estimator:
  pi1tilde <- rep(mean(pi1hat), S)

  ##Wald test
  ##---------

  if( doWaldTest ){

    ##build mAhat:
    mAhat <- matrix(0, S, S)
    for(j in 1:S){
      term1 <- (1-mIt[,j])/((1-pi1hat[j])^2)
      term2 <- mIt[,j]/(pi1hat[j]^2)
      mAhat[j,j] <- mean(term1 + term2)
    }
    mAhatInv <- solve(mAhat)

    ##compute scores:
    mShat <- matrix(NA, iT, S) #matrix of scores
    for(j in 1:S){
      term1 <- (1-mIt[,j])/(1-pi1hat[j])
      term2 <- mIt[,j]/pi1hat[j]
      mShat[,j] <- term1 - term2
    }

    ##compute the "meat" (B-matrix) mBhat:
    mGhat0 <- crossprod(mShat)/iT  #Gamma(j=0)
    mBhat <- mGhat0
    if( iL > 0 ){
      for(j in 1:iL){
        mS0 <- cbind(mShat[-c(1:j),])
        mSj <- cbind(mShat[-c(c(iT-j+1):iT),])
        mGhatj1 <- crossprod(mS0,mSj)/iT
        mGhatj2 <- crossprod(mSj,mS0)/iT
        mBhat <- mBhat + vW[j] * (mGhatj1 + mGhatj2)
      }
    }

    ##compute vcov:
    mSigmahat <- mAhatInv %*% mBhat %*% mAhatInv

    ##compute statistic, record p-value:
    tmp <- mR %*% pi1hat
    Statistic <-
      iT * t(tmp) %*% solve(mR %*% mSigmahat %*% t(mR)) %*% tmp
    Statistic <- DFwaldAdj * Statistic ##DF-adjust
    pvalue <- pchisq(Statistic, S-1, lower.tail=FALSE) ##p-value

  } #close doWaldTest

  ##Lagrange Multiplier test
  ##------------------------

  if( doLMtest ){

    ##build mAtilde:
    mAtilde <- matrix(0, S, S)
    for(j in 1:S){
      term1 <- (1-mIt[,j])/((1-pi1tilde[j])^2)
      term2 <- mIt[,j]/(pi1tilde[j]^2)
      mAtilde[j,j] <- mean(term1 + term2)
    }
    mAtildeInv <- solve(mAtilde)

    ##compute scores:
    mStilde <- matrix(NA, iT, S) #matrix of scores
    for(j in 1:S){
      term1 <- (1-mIt[,j])/(1-pi1tilde[j])
      term2 <- mIt[,j]/pi1tilde[j]
      mStilde[,j] <- term1 - term2
    }

    ##compute the "meat" (B-matrix) mBtilde:
    mGtilde0 <- crossprod(mStilde)/iT  #Gamma(j=0)
    mBtilde <- mGtilde0
    if( iL > 0 ){
      for(j in 1:iL){
        mS0 <- cbind(mStilde[-c(1:j),])
        mSj <- cbind(mStilde[-c(c(iT-j+1):iT),])
        mGtildej1 <- crossprod(mS0,mSj)/iT
        mGtildej2 <- crossprod(mSj,mS0)/iT
        mBtilde <- mBtilde + vW[j] * (mGtildej1 + mGtildej2)
      }
    }

    ##compute statistic, record p-value:
    mStildeSums <- matrix(NA, S, 1)
    mStildeSums[,1] <- colSums(mStilde)
    mStildeSumst <- t(mStildeSums)
    term1 <- (1/iT) * mStildeSumst %*% mAtildeInv %*% mRt
    term2 <- solve(mR %*% mAtildeInv %*% mBtilde %*% mAtildeInv %*% mRt)
    term3 <- mR %*% mAtildeInv %*% mStildeSums
    Statistic <- term1 %*% term2 %*% term3
    Statistic <- DFlmAdj * Statistic ##DF-adjust
    pvalue <- pchisq(Statistic, S-1, lower.tail=FALSE) ##p-value

  } #close doLMtest


  ##return result
  ##-------------

  result <- list()
  result$S <- S
  result$T <- iT
  result$type <- type
  result$df.adjustment <- df.adjustment
  result$bandwidth <- iL
  result$statistic <- Statistic
  result$pvalue <- pvalue
  return(result)

} #close doTest() function
