###########################################################
## This is an R file part of the project:
##
##   "Testing the zero-process of intraday financial
##    returns for non-stationary periodicity"
##
## by Ovidijus Stauskas and Genaro Sucarrat
##
## This R file analyses the intraday stock data.
##
## First created 26 April 2023, Madrid.
##
## Contents:
##
## 1 INITIATE
## 2 AUXILIARY FUNCTION
## 3 ONE-MINUTE RETURNS
## 4 FIVE-MINUTE RETURNS
## 5 FIFTEEN-MINUTE RETURNS
## 6 THIRTY-MINUTE RETURNS
## 7 HOURLY RETURNS
## 8 PRINT RESULTS FOR PAPER
##
###########################################################


###########################################################
##1 INITIATE
###########################################################

##clean workspace:                         
rm(list=ls())

##set work directory (i.e. doTest-source.R and folder 'data' are):
myDir <- choose.dir()
setwd(myDir)

##load source:
source("doTest-source.R")

##load packages on CRAN:
library(zoo)
library(xts)
library(gets) #needed for dropvar() function
library(xtable)


###########################################################
## 2 AUXILIARY FUNCTION
###########################################################

## iDFlmFun function:
##===================

## this function computes the value used in the DF-adjustment of
## the DF-adjusted LM-statistic. Function argument:
##
##  S: number of periods
##
## value returned: a numeric, positive or negative

iDFlmFun <- function(S){
  sxfun <- function(S, location, speed){
    1/(1+exp( - speed * ( S - location ) ) )
  }
  result <- 60 +  20*sxfun(S, 8-4, 1) - 20*sxfun(S, 16-2, 0.5) -
    20*sxfun(S, 32-2, 0.5) - 20*sxfun(S, 48, 0.2) - 20*sxfun(S, 96-36, 0.2)
  return(result)
}


###########################################################
## 3 ONE-MINUTE RETURNS
###########################################################

##create/prepare 1-min list:
##==========================

list1min <- list()
tickers <- c("AMZN", "FB", "MSFT","TSLA","IBM")

##create vector with 1-min stamps:
##================================

oneMinStamps <- NULL
for(i in 0:23){
  tmp <- i*10000 + seq(0,59)*100
  oneMinStamps <- rbind(oneMinStamps, tmp)
}
oneMinStamps <- as.vector( t(oneMinStamps) )
t1 <- which(oneMinStamps == 40000) #first trade
t2 <- which(oneMinStamps == 195900) #last trade
oneMinStamps <- oneMinStamps[t1:t2]

##import transformed data:
##========================

tmpnames <- paste0(getwd(), "/data/", tickers, "-1-minute-prices.csv")
for(i in 1:length(tickers)){
  tmpdata <- as.matrix( read.csv(tmpnames[i], row.names=1) )
  colnames(tmpdata) <- format(oneMinStamps, trim=TRUE)
  list1min[[i]] <- tmpdata
}
names(list1min) <- tickers

##create list of non-zero indicators
##==================================

list1minIt <- list1min
for(i in 1:length(tickers)){
  tmpdata <- list1min[[i]]
  tmpdataIt <- tmpdata
  for(j in 1:NROW(tmpdata)){
    for(k in 1:NCOL(tmpdata)){
      if( j==1 && k==1 ){
        tmpdataIt[j,k] <- 1
      }else{
        if( k==1 ){
          tmpdataIt[j,k] <-
            ifelse(tmpdata[j,k] == tmpdata[c(j-1),NCOL(tmpdata)], 0, 1)
        }
        if( k>1 ){
          tmpdataIt[j,k] <-
            ifelse(tmpdata[j,k] == tmpdata[j,c(k-1)], 0, 1)
        }
      }
    } #close for(k)
  } #close for(j)
  list1minIt[[i]] <- tmpdataIt
} #close for(i)


##descriptive statistics (all trading hours)
##==========================================

descstat1min <- matrix(NA, length(tickers), 8)
colnames(descstat1min) <-
  c("n", "0s", "pi0", "rho1", "T", "maxrho1", "minrho1", "S")
rownames(descstat1min) <- tickers
for(i in 1:NROW(descstat1min)){
  tmpdata <- list1minIt[[i]]
  Ii <- as.vector(t(tmpdata))

  descstat1min[i,"n"] <- length(Ii)
  descstat1min[i,"0s"] <- length(which(Ii==0))
  descstat1min[i,"pi0"] <- descstat1min[i,"0s"]/descstat1min[i,"n"]
  descstat1min[i,"rho1"] <- acf(Ii, plot=FALSE)$acf[2]
  descstat1min[i,"T"] <- NROW(tmpdata)
  descstat1min[i,"S"] <- NCOL(tmpdata)

  rho1st <- rep(NA, NCOL(tmpdata))
  for(j in 1:NCOL(tmpdata)){
    It <- tmpdata[,j]
    rho1st[j] <- acf(It, lag.max=1, plot=FALSE)$acf[2]
  }
  descstat1min[i,"maxrho1"] <- max(abs(rho1st), na.rm=TRUE)
  descstat1min[i,"minrho1"] <- min(abs(rho1st), na.rm=TRUE)
}


##do tests (all trading hours)
##============================

##unadjusted tests:
df1minTests <- as.data.frame(NULL)
tmprow <- matrix(NA, 1, 7)
colnames(tmprow) <- c("ticker", "S", "T", "type", "bandwidth",
  "statistic", "pvalue") 
tmprow <- as.data.frame(tmprow)
#rownames(tmprow) <- ""

##df-adjusted tests:
df1minTestsAdj <- as.data.frame(NULL)
tmprowadj <- matrix(NA, 1, 8)
colnames(tmprowadj) <- c("ticker", "S", "T", "type", "df.adjustment",
  "bandwidth", "statistic", "pvalue") 
tmprowadj <- as.data.frame(tmprowadj)
#rownames(tmprowadj) <- ""

##loop:
for(i in 1:length(list1minIt)){
  
  ##S=all
  ##-----
  
  ##build mIt
  mIt <- list1minIt[[i]]
  mIt <- dropvar(mIt)
  
  ##unadjusted LM-test:
  tmp <- doTest(mIt, type="lm", df.adjustment=NULL, hac=TRUE)
  tmprow[1] <- names(list1minIt)[i]
  for(j in 1:length(tmp)){ tmprow[c(j+1)] <- tmp[[j]] }
  df1minTests <- rbind(df1minTests, tmprow)
      
  ##adjusted LM-test:
  S <- NCOL(mIt) ##for DF-adjustment
  iDFlm <- iDFlmFun(S)
  DFlmAdj <- 1 - iDFlm/NROW(mIt)
  DFlmAdj <- max(0.1, DFlmAdj)
  DFlmAdj
  tmp <- doTest(mIt, type="lm", df.adjustment=DFlmAdj, hac=TRUE)
  tmprowadj[1] <- names(list1minIt)[i]
  for(j in 1:length(tmp)){ tmprowadj[c(j+1)] <- tmp[[j]] }
  df1minTestsAdj <- rbind(df1minTestsAdj, tmprowadj)
  
  ##S=every third
  ##-------------
  
  ##re-build mIt:
  mIt <- mIt[,seq(1,NCOL(mIt),by=3)] #select every third
  
  ##unadjusted LM-test:
  tmp <- doTest(mIt, type="lm", df.adjustment=NULL, hac=TRUE)
  tmprow[1] <- names(list1minIt)[i]
  for(j in 1:length(tmp)){ tmprow[c(j+1)] <- tmp[[j]] }
  df1minTests <- rbind(df1minTests, tmprow)
      
  ##adjusted LM-test:
  S <- NCOL(mIt) ##for DF-adjustment
  iDFlm <- iDFlmFun(S)
  DFlmAdj <- 1 - iDFlm/NROW(mIt)
  DFlmAdj <- max(0.1, DFlmAdj)
  DFlmAdj
  tmp <- doTest(mIt, type="lm", df.adjustment=DFlmAdj, hac=TRUE)
  tmprowadj[1] <- names(list1minIt)[i]
  for(j in 1:length(tmp)){ tmprowadj[c(j+1)] <- tmp[[j]] }
  df1minTestsAdj <- rbind(df1minTestsAdj, tmprowadj)

  ##S=every sixth
  ##-------------
  
  ##re-build mIt:
  mIt <- mIt[,seq(1,NCOL(mIt),by=3)] #select every third again
  
  ##unadjusted LM-test:
  tmp <- doTest(mIt, type="lm", df.adjustment=NULL, hac=TRUE)
  tmprow[1] <- names(list1minIt)[i]
  for(j in 1:length(tmp)){ tmprow[c(j+1)] <- tmp[[j]] }
  df1minTests <- rbind(df1minTests, tmprow)
      
  ##adjusted LM-test:
  S <- NCOL(mIt) ##for DF-adjustment
  iDFlm <- iDFlmFun(S)
  DFlmAdj <- 1 - iDFlm/NROW(mIt)
  DFlmAdj <- max(0.1, DFlmAdj)
  DFlmAdj
  tmp <- doTest(mIt, type="lm", df.adjustment=DFlmAdj, hac=TRUE)
  tmprowadj[1] <- names(list1minIt)[i]
  for(j in 1:length(tmp)){ tmprowadj[c(j+1)] <- tmp[[j]] }
  df1minTestsAdj <- rbind(df1minTestsAdj, tmprowadj)

} #close for(i) loop


##descriptive statistics (main trading hours)
##===========================================

descstat1minMain <- matrix(NA, length(tickers), 8)
colnames(descstat1minMain) <-
  c("n", "0s", "pi0", "rho1", "T", "maxrho1", "minrho1", "S")
rownames(descstat1minMain) <- tickers
for(i in 1:NROW(descstat1minMain)){
  tmpdata <- list1minIt[[i]]
  t1 <- which(colnames(tmpdata)=="93000")
  t2 <- which(colnames(tmpdata)=="160000")
  tmpdata <- tmpdata[,t1:t2]
  Ii <- as.vector(t(tmpdata))

  descstat1minMain[i,"n"] <- length(Ii)
  descstat1minMain[i,"0s"] <- length(which(Ii==0))
  descstat1minMain[i,"pi0"] <-
    descstat1minMain[i,"0s"]/descstat1minMain[i,"n"]
  descstat1minMain[i,"rho1"] <- acf(Ii, plot=FALSE)$acf[2]
  descstat1minMain[i,"T"] <- NROW(tmpdata)
  descstat1minMain[i,"S"] <- NCOL(tmpdata)

  rho1st <- rep(NA, NCOL(tmpdata))
  for(j in 1:NCOL(tmpdata)){
    It <- tmpdata[,j]
    rho1st[j] <- acf(It, lag.max=1, plot=FALSE)$acf[2]
  }
  descstat1minMain[i,"maxrho1"] <- max(abs(rho1st), na.rm=TRUE)
  descstat1minMain[i,"minrho1"] <- min(abs(rho1st), na.rm=TRUE)
}


##do tests (main trading hours)
##=============================

##unadjusted tests:
df1minTestsMain <- as.data.frame(NULL)
tmprow <- matrix(NA, 1, 7)
colnames(tmprow) <- c("ticker", "S", "T", "type", "bandwidth",
  "statistic", "pvalue") 
tmprow <- as.data.frame(tmprow)

##df-adjusted tests:
df1minTestsMainAdj <- as.data.frame(NULL)
tmprowadj <- matrix(NA, 1, 8)
colnames(tmprowadj) <- c("ticker", "S", "T", "type", "df.adjustment",
  "bandwidth", "statistic", "pvalue") 
tmprowadj <- as.data.frame(tmprowadj)

##loop:
for(i in 1:length(list1minIt)){
  
  ##S=all
  ##-----
  
  ##build mIt
  mIt <- list1minIt[[i]]
  t1 <- which(colnames(mIt)=="93000")
  t2 <- which(colnames(mIt)=="160000")
  mIt <- mIt[,t1:t2]
  mIt <- dropvar(mIt)
  
  ##unadjusted LM-test:
  tmp <- doTest(mIt, type="lm", df.adjustment=NULL, hac=TRUE)
  tmprow[1] <- names(list1minIt)[i]
  for(j in 1:length(tmp)){ tmprow[c(j+1)] <- tmp[[j]] }
  df1minTestsMain <- rbind(df1minTestsMain, tmprow)
      
  ##adjusted LM-test:
  S <- NCOL(mIt) ##for DF-adjustment
  iDFlm <- iDFlmFun(S)
  DFlmAdj <- 1 - iDFlm/NROW(mIt)
  DFlmAdj <- max(0.1, DFlmAdj)
  DFlmAdj
  tmp <- doTest(mIt, type="lm", df.adjustment=DFlmAdj, hac=TRUE)
  tmprowadj[1] <- names(list1minIt)[i]
  for(j in 1:length(tmp)){ tmprowadj[c(j+1)] <- tmp[[j]] }
  df1minTestsMainAdj <- rbind(df1minTestsMainAdj, tmprowadj)
  
  ##S=every third
  ##-------------
  
  ##re-build mIt:
  mItShort <- mIt[,seq(1,NCOL(mIt),by=3)] #select every third
  
  ##unadjusted LM-test:
  tmp <- doTest(mItShort, type="lm", df.adjustment=NULL, hac=TRUE)
  tmprow[1] <- names(list1minIt)[i]
  for(j in 1:length(tmp)){ tmprow[c(j+1)] <- tmp[[j]] }
  df1minTestsMain <- rbind(df1minTestsMain, tmprow)
      
  ##adjusted LM-test:
  S <- NCOL(mItShort) ##for DF-adjustment
  iDFlm <- iDFlmFun(S)
  DFlmAdj <- 1 - iDFlm/NROW(mItShort)
  DFlmAdj <- max(0.1, DFlmAdj)
  DFlmAdj
  tmp <- doTest(mItShort, type="lm", df.adjustment=DFlmAdj, hac=TRUE)
  tmprowadj[1] <- names(list1minIt)[i]
  for(j in 1:length(tmp)){ tmprowadj[c(j+1)] <- tmp[[j]] }
  df1minTestsMainAdj <- rbind(df1minTestsMainAdj, tmprowadj)

  ##S=every sixth
  ##-------------
  
  ##re-build mIt:
  mItShort <- mIt[,seq(1,NCOL(mIt),by=6)] #select every third again
  
  ##unadjusted LM-test:
  tmp <- doTest(mItShort, type="lm", df.adjustment=NULL, hac=TRUE)
  tmprow[1] <- names(list1minIt)[i]
  for(j in 1:length(tmp)){ tmprow[c(j+1)] <- tmp[[j]] }
  df1minTestsMain <- rbind(df1minTestsMain, tmprow)
      
  ##adjusted LM-test:
  S <- NCOL(mItShort) ##for DF-adjustment
  iDFlm <- iDFlmFun(S)
  DFlmAdj <- 1 - iDFlm/NROW(mItShort)
  DFlmAdj <- max(0.1, DFlmAdj)
  DFlmAdj
  tmp <- doTest(mItShort, type="lm", df.adjustment=DFlmAdj, hac=TRUE)
  tmprowadj[1] <- names(list1minIt)[i]
  for(j in 1:length(tmp)){ tmprowadj[c(j+1)] <- tmp[[j]] }
  df1minTestsMainAdj <- rbind(df1minTestsMainAdj, tmprowadj)

} #close for(i) loop


##intraday plot of zero-probabilities:
##====================================

windows(5,3)
  
##get current par-values:
def.par <- par(no.readonly=TRUE)

##margins (default=5.1,4.1,4.1,2.1):
par(mar = c(4,3.5,2,1.5)) #distance from: bottom,left,top,right

##make plots:
for(i in 1:length(list1minIt)){
  
  ##intraday zero-probability::
  m1minIt <- list1minIt[[i]]
  pi0t <- 1 - colMeans(m1minIt)
  assetName <- names(list1minIt)[i]
  
  ##create plot:
  mainTxt <- paste0(assetName, ":")
  plot(1:length(pi0t), pi0t, lwd=1, ylim=c(-0.02,1.08), col="white", 
    xaxt="n", #suppress ticks on x-axis
    yaxt="n", #suppress ticks on y-axis
    cex.main=1.5,
    main=mainTxt,
    cex.lab=1.2,
    xlab="",
    ylab="")
  title(line=2,
    xlab="1-minute interval (S = 960)", ylab="Zero-probability")
  lines(as.zoo(pi0t), col="blue", lwd=1)
  
  ##y-axis:
  abline(h=seq(0,1,by=0.2), col="grey", lty="dotted")
  axis(side=2, at=seq(0,1,by=0.2), padj=0.4,
    labels=c("0.0","0.2","0.4","0.6","0.8","1.0"),
    cex.axis=1.1)
  
  ##x-axis
  xwhere <- seq(1,960,by=120)
  xwhere <- c(xwhere,960)
  axis(side=1, at=xwhere, labels=xwhere, cex.axis=1.1, padj=-0.6)
  whichStart <- which( colnames(m1minIt)=="93000" )
  whichEnd <- which( colnames(m1minIt)=="160000" )
  xwhere <- c(1, whichStart, whichEnd, 961) #4h,9h30,16h,20h
  abline(v=xwhere, col="grey")
  text(c(45,410,722), 1.06, adj=0, col="grey", cex=1,
    labels=c("04:00-09:30", "09:30-16:00", "16:00-20:00"))
  
  ##redo lines:
  lines(as.zoo(pi0t), col="blue", lwd=1)
  
  ##save plot:
  desttxt <- paste0("./figures/STOCK-1min-", assetName, ".pdf")
#  savePlot(desttxt, type="pdf")

} #close for(i) loop

##return to old par-values:
par(def.par)

##turn off graphics:
graphics.off()


############################################################
## 4 FIVE-MINUTE RETURNS
############################################################

##create 5-minute timestamps:
whichOnes <- seq(1,length(oneMinStamps),by=5)
fiveMinStamps <- oneMinStamps[whichOnes]

##create list of 5-min values
##===========================

list5min <- list1min

##fill the list:
for(i in 1:length(tickers)){
  tmpdata <- list1min[[i]]
  tmpdata <- tmpdata[,whichOnes]
  list5min[[i]] <- tmpdata
}

##create list of non-zero indicators
##==================================

list5minIt <- list5min
for(i in 1:length(tickers)){
  tmpdata <- list5min[[i]]
  tmpdataIt <- tmpdata
  for(j in 1:NROW(tmpdata)){
    for(k in 1:NCOL(tmpdata)){
      if( j==1 && k==1 ){
        tmpdataIt[j,k] <- 1
      }else{
        if( k==1 ){
          tmpdataIt[j,k] <-
            ifelse(tmpdata[j,k] == tmpdata[c(j-1),NCOL(tmpdata)], 0, 1)
        }
        if( k>1 ){
          tmpdataIt[j,k] <-
            ifelse(tmpdata[j,k] == tmpdata[j,c(k-1)], 0, 1)
        }
      }
    } #close for(k)
  } #close for(j)
  list5minIt[[i]] <- tmpdataIt
} #close for(i)


##descriptive statistics (all trading hours)
##==========================================

descstat5min <- matrix(NA, length(tickers), 8)
colnames(descstat5min) <-
  c("n", "0s", "pi0", "rho1", "T", "maxrho1", "minrho1", "S")
rownames(descstat5min) <- tickers
for(i in 1:NROW(descstat5min)){
  tmpdata <- list5minIt[[i]]
  Ii <- as.vector(t(tmpdata))

  descstat5min[i,"n"] <- length(Ii)
  descstat5min[i,"0s"] <- length(which(Ii==0))
  descstat5min[i,"pi0"] <- descstat5min[i,"0s"]/descstat5min[i,"n"]
  descstat5min[i,"rho1"] <- acf(Ii, plot=FALSE)$acf[2]
  descstat5min[i,"T"] <- NROW(tmpdata)
  descstat5min[i,"S"] <- NCOL(tmpdata)

  rho1st <- rep(NA, NCOL(tmpdata))
  for(j in 1:NCOL(tmpdata)){
    It <- tmpdata[,j]
    rho1st[j] <- acf(It, lag.max=1, plot=FALSE)$acf[2]
  }
  descstat5min[i,"maxrho1"] <- max(abs(rho1st), na.rm=TRUE)
  descstat5min[i,"minrho1"] <- min(abs(rho1st), na.rm=TRUE)
}


##do tests (all trading hours)
##============================

##unadjusted tests:
df5minTests <- as.data.frame(NULL)
tmprow <- matrix(NA, 1, 7)
colnames(tmprow) <- c("ticker", "S", "T", "type", "bandwidth",
  "statistic", "pvalue") 
tmprow <- as.data.frame(tmprow)

##df-adjusted tests:
df5minTestsAdj <- as.data.frame(NULL)
tmprowadj <- matrix(NA, 1, 8)
colnames(tmprowadj) <- c("ticker", "S", "T", "type", "df.adjustment",
  "bandwidth", "statistic", "pvalue") 
tmprowadj <- as.data.frame(tmprowadj)

##loop:
for(i in 1:length(list5minIt)){
  
  ##S=all
  ##-----
  
  ##build mIt
  mIt <- list5minIt[[i]]
  mIt <- dropvar(mIt)
  
  ##unadjusted LM-test:
  tmp <- doTest(mIt, type="lm", df.adjustment=NULL, hac=TRUE)
  tmprow[1] <- names(list5minIt)[i]
  for(j in 1:length(tmp)){ tmprow[c(j+1)] <- tmp[[j]] }
  df5minTests <- rbind(df5minTests, tmprow)
      
  ##adjusted LM-test:
  S <- NCOL(mIt) ##for DF-adjustment
  iDFlm <- iDFlmFun(S)
  DFlmAdj <- 1 - iDFlm/NROW(mIt)
  DFlmAdj <- max(0.1, DFlmAdj)
  DFlmAdj
  tmp <- doTest(mIt, type="lm", df.adjustment=DFlmAdj, hac=TRUE)
  tmprowadj[1] <- names(list5minIt)[i]
  for(j in 1:length(tmp)){ tmprowadj[c(j+1)] <- tmp[[j]] }
  df5minTestsAdj <- rbind(df5minTestsAdj, tmprowadj)
  
  ##S=every third
  ##-------------
  
  ##build mIt:
  mIt <- mIt[,seq(1,NCOL(mIt),by=3)] #select every third
  
  ##unadjusted LM-test:
  tmp <- doTest(mIt, type="lm", df.adjustment=NULL, hac=TRUE)
  tmprow[1] <- names(list5minIt)[i]
  for(j in 1:length(tmp)){ tmprow[c(j+1)] <- tmp[[j]] }
  df5minTests <- rbind(df5minTests, tmprow)
      
  ##adjusted LM-test:
  S <- NCOL(mIt) ##for DF-adjustment
  iDFlm <- iDFlmFun(S)
  DFlmAdj <- 1 - iDFlm/NROW(mIt)
  DFlmAdj <- max(0.1, DFlmAdj)
  DFlmAdj
  tmp <- doTest(mIt, type="lm", df.adjustment=DFlmAdj, hac=TRUE)
  tmprowadj[1] <- names(list5minIt)[i]
  for(j in 1:length(tmp)){ tmprowadj[c(j+1)] <- tmp[[j]] }
  df5minTestsAdj <- rbind(df5minTestsAdj, tmprowadj)

} #close for(i) loop


##descriptive statistics (main trading hours)
##===========================================

descstat5minMain <- matrix(NA, length(tickers), 8)
colnames(descstat5minMain) <-
  c("n", "0s", "pi0", "rho1", "T", "maxrho1", "minrho1", "S")
rownames(descstat5minMain) <- tickers
for(i in 1:NROW(descstat5minMain)){
  tmpdata <- list5minIt[[i]]
  t1 <- which(colnames(tmpdata)=="93000")
  t2 <- which(colnames(tmpdata)=="160000")
  tmpdata <- tmpdata[,t1:t2]
  Ii <- as.vector(t(tmpdata))

  descstat5minMain[i,"n"] <- length(Ii)
  descstat5minMain[i,"0s"] <- length(which(Ii==0))
  descstat5minMain[i,"pi0"] <- descstat5minMain[i,"0s"]/descstat5minMain[i,"n"]
  descstat5minMain[i,"rho1"] <- acf(Ii, plot=FALSE)$acf[2]
  descstat5minMain[i,"T"] <- NROW(tmpdata)
  descstat5minMain[i,"S"] <- NCOL(tmpdata)

  rho1st <- rep(NA, NCOL(tmpdata))
  for(j in 1:NCOL(tmpdata)){
    It <- tmpdata[,j]
    rho1st[j] <- acf(It, lag.max=1, plot=FALSE)$acf[2]
  }
  descstat5minMain[i,"maxrho1"] <- max(abs(rho1st), na.rm=TRUE)
  descstat5minMain[i,"minrho1"] <- min(abs(rho1st), na.rm=TRUE)
}


##do tests (main trading hours)
##=============================

##unadjusted tests:
df5minTestsMain <- as.data.frame(NULL)
tmprow <- matrix(NA, 1, 7)
colnames(tmprow) <- c("ticker", "S", "T", "type", "bandwidth",
  "statistic", "pvalue") 
tmprow <- as.data.frame(tmprow)

##df-adjusted tests:
df5minTestsMainAdj <- as.data.frame(NULL)
tmprowadj <- matrix(NA, 1, 8)
colnames(tmprowadj) <- c("ticker", "S", "T", "type", "df.adjustment",
  "bandwidth", "statistic", "pvalue") 
tmprowadj <- as.data.frame(tmprowadj)

##loop:
for(i in 1:length(list5minIt)){
  
  ##S=all
  ##-----
  
  ##build mIt
  mIt <- list5minIt[[i]]
  t1 <- which(colnames(mIt)=="93000")
  t2 <- which(colnames(mIt)=="160000")
  mIt <- mIt[,t1:t2]
  mIt <- dropvar(mIt)
  
  ##unadjusted LM-test:
  tmp <- doTest(mIt, type="lm", df.adjustment=NULL, hac=TRUE)
  tmprow[1] <- names(list5minIt)[i]
  for(j in 1:length(tmp)){ tmprow[c(j+1)] <- tmp[[j]] }
  df5minTestsMain <- rbind(df5minTestsMain, tmprow)
      
  ##adjusted LM-test:
  S <- NCOL(mIt) ##for DF-adjustment
  iDFlm <- iDFlmFun(S)
  DFlmAdj <- 1 - iDFlm/NROW(mIt)
  DFlmAdj <- max(0.1, DFlmAdj)
  DFlmAdj
  tmp <- doTest(mIt, type="lm", df.adjustment=DFlmAdj, hac=TRUE)
  tmprowadj[1] <- names(list5minIt)[i]
  for(j in 1:length(tmp)){ tmprowadj[c(j+1)] <- tmp[[j]] }
  df5minTestsMainAdj <- rbind(df5minTestsMainAdj, tmprowadj)
  
  ##S=every third
  ##-------------
  
  ##build mIt:
  mIt <- mIt[,seq(1,NCOL(mIt),by=3)] #select every third
  
  ##unadjusted LM-test:
  tmp <- doTest(mIt, type="lm", df.adjustment=NULL, hac=TRUE)
  tmprow[1] <- names(list5minIt)[i]
  for(j in 1:length(tmp)){ tmprow[c(j+1)] <- tmp[[j]] }
  df5minTestsMain <- rbind(df5minTestsMain, tmprow)
      
  ##adjusted LM-test:
  S <- NCOL(mIt) ##for DF-adjustment
  iDFlm <- iDFlmFun(S)
  DFlmAdj <- 1 - iDFlm/NROW(mIt)
  DFlmAdj <- max(0.1, DFlmAdj)
  DFlmAdj
  tmp <- doTest(mIt, type="lm", df.adjustment=DFlmAdj, hac=TRUE)
  tmprowadj[1] <- names(list5minIt)[i]
  for(j in 1:length(tmp)){ tmprowadj[c(j+1)] <- tmp[[j]] }
  df5minTestsMainAdj <- rbind(df5minTestsMainAdj, tmprowadj)

} #close for(i) loop


##intraday plot of zero-probabilities:
##====================================

windows(5,3)
  
##get current par-values:
def.par <- par(no.readonly=TRUE)

##margins (default=5.1,4.1,4.1,2.1):
par(mar = c(4,3.5,2,1.5)) #distance from: bottom,left,top,right

##make plots:
for(i in 1:length(list1minIt)){
  
  ##intraday zero-probability::
  m5minIt <- list5minIt[[i]]
  pi0t <- 1 - colMeans(m5minIt)
  assetName <- names(list5minIt)[i]
  
  ##create plot:
  mainTxt <- paste0(assetName, ":")
  plot(1:length(pi0t), pi0t, lwd=1, ylim=c(-0.02,1.08), col="white", 
    xaxt="n", #suppress ticks on x-axis
    yaxt="n", #suppress ticks on y-axis
    cex.main=1.5,
    main=mainTxt,
    cex.lab=1.2,
    xlab="",
    ylab="")
  title(line=2,
    xlab=paste0("5-minute interval (S = ", ncol(m5minIt), ")"),
    ylab="Zero-probability")
  lines(as.zoo(pi0t), col="blue", lwd=1)
  
  ##y-axis:
  abline(h=seq(0,1,by=0.2), col="grey", lty="dotted")
  axis(side=2, at=seq(0,1,by=0.2), padj=0.4,
    labels=c("0.0","0.2","0.4","0.6","0.8","1.0"),
    cex.axis=1.1)
  
  ##x-axis
  xwhere <- seq(1,192,by=24)
  xwhere <- c(xwhere,192)
  axis(side=1, at=xwhere, labels=xwhere, cex.axis=1.1, padj=-0.6)
  whichStart <- which( colnames(m5minIt)=="93000" )
  whichEnd <- which( colnames(m5minIt)=="160000" )
  xwhere <- c(1, whichStart, whichEnd, 192) #4h,9h30,16h,20h
  abline(v=xwhere, col="grey")
  text(c(10,83,145.5), 1.06, adj=0, col="grey", cex=1,
    labels=c("04:00-09:30", "09:30-16:00", "16:00-20:00"))
  
  ##redo lines:
  lines(as.zoo(pi0t), col="blue", lwd=1)
  
  ##save plot:
  desttxt <- paste0("./figures/STOCK-5min-", assetName, ".pdf")
#  savePlot(desttxt, type="pdf")

} #close for(i) loop

##return to old par-values:
par(def.par)

##turn off graphics:
graphics.off()


############################################################
## 5 FIFTEEN-MINUTE RETURNS
############################################################

##create 15-minute timestamps:
whichOnes <- seq(1,length(oneMinStamps),by=15)
fifteenMinStamps <- oneMinStamps[whichOnes]

##create list of 15-min values
##===========================

list15min <- list1min

##fill the list:
for(i in 1:length(tickers)){
  tmpdata <- list1min[[i]]
  tmpdata <- tmpdata[,whichOnes]
  list15min[[i]] <- tmpdata
}

##create list of non-zero indicators
##==================================

list15minIt <- list15min
for(i in 1:length(tickers)){
  tmpdata <- list15min[[i]]
  tmpdataIt <- tmpdata
  for(j in 1:NROW(tmpdata)){
    for(k in 1:NCOL(tmpdata)){
      if( j==1 && k==1 ){
        tmpdataIt[j,k] <- 1
      }else{
        if( k==1 ){
          tmpdataIt[j,k] <-
            ifelse(tmpdata[j,k] == tmpdata[c(j-1),NCOL(tmpdata)], 0, 1)
        }
        if( k>1 ){
          tmpdataIt[j,k] <-
            ifelse(tmpdata[j,k] == tmpdata[j,c(k-1)], 0, 1)
        }
      }
    } #close for(k)
  } #close for(j)
  list15minIt[[i]] <- tmpdataIt
} #close for(i)


##descriptive statistics (all trading hours)
##==========================================

descstat15min <- matrix(NA, length(tickers), 8)
colnames(descstat15min) <-
  c("n", "0s", "pi0", "rho1", "T", "maxrho1", "minrho1", "S")
rownames(descstat15min) <- tickers
for(i in 1:NROW(descstat15min)){
  tmpdata <- list15minIt[[i]]
  Ii <- as.vector(t(tmpdata))

  descstat15min[i,"n"] <- length(Ii)
  descstat15min[i,"0s"] <- length(which(Ii==0))
  descstat15min[i,"pi0"] <- descstat15min[i,"0s"]/descstat15min[i,"n"]
  descstat15min[i,"rho1"] <- acf(Ii, plot=FALSE)$acf[2]
  descstat15min[i,"T"] <- NROW(tmpdata)
  descstat15min[i,"S"] <- NCOL(tmpdata)

  rho1st <- rep(NA, NCOL(tmpdata))
  for(j in 1:NCOL(tmpdata)){
    It <- tmpdata[,j]
    rho1st[j] <- acf(It, lag.max=1, plot=FALSE)$acf[2]
  }
  descstat15min[i,"maxrho1"] <- max(abs(rho1st), na.rm=TRUE)
  descstat15min[i,"minrho1"] <- min(abs(rho1st), na.rm=TRUE)
}


##do tests (all trading hours)
##============================

##unadjusted tests:
df15minTests <- as.data.frame(NULL)
tmprow <- matrix(NA, 1, 7)
colnames(tmprow) <- c("ticker", "S", "T", "type", "bandwidth",
  "statistic", "pvalue") 
tmprow <- as.data.frame(tmprow)
#rownames(tmprow) <- ""

##df-adjusted tests:
df15minTestsAdj <- as.data.frame(NULL)
tmprowadj <- matrix(NA, 1, 8)
colnames(tmprowadj) <- c("ticker", "S", "T", "type", "df.adjustment",
  "bandwidth", "statistic", "pvalue") 
tmprowadj <- as.data.frame(tmprowadj)
#rownames(tmprowadj) <- ""

##loop:
for(i in 1:length(list15minIt)){
  
  ##S=all
  ##-----
  
  ##build mIt
  mIt <- list15minIt[[i]]
  mIt <- dropvar(mIt)
  
  ##unadjusted LM-test:
  tmp <- doTest(mIt, type="lm", df.adjustment=NULL, hac=TRUE)
  tmprow[1] <- tickers[i]
  for(j in 1:length(tmp)){ tmprow[c(j+1)] <- tmp[[j]] }
  df15minTests <- rbind(df15minTests, tmprow)
      
  ##adjusted LM-test:
  S <- NCOL(mIt) ##for DF-adjustment
  iDFlm <- iDFlmFun(S)
  DFlmAdj <- 1 - iDFlm/NROW(mIt)
  DFlmAdj <- max(0.1, DFlmAdj)
  DFlmAdj
  tmp <- doTest(mIt, type="lm", df.adjustment=DFlmAdj, hac=TRUE)
  tmprowadj[1] <- tickers[i]
  for(j in 1:length(tmp)){ tmprowadj[c(j+1)] <- tmp[[j]] }
  df15minTestsAdj <- rbind(df15minTestsAdj, tmprowadj)

} #close for(i) loop


##descriptive statistics (main trading hours)
##===========================================

descstat15minMain <- matrix(NA, length(tickers), 8)
colnames(descstat15minMain) <-
  c("n", "0s", "pi0", "rho1", "T", "maxrho1", "minrho1", "S")
rownames(descstat15minMain) <- tickers
for(i in 1:NROW(descstat15minMain)){
  tmpdata <- list15minIt[[i]]
  t1 <- which(colnames(tmpdata)=="93000")
  t2 <- which(colnames(tmpdata)=="160000")
  tmpdata <- tmpdata[,t1:t2]
  Ii <- as.vector(t(tmpdata))

  descstat15minMain[i,"n"] <- length(Ii)
  descstat15minMain[i,"0s"] <- length(which(Ii==0))
  descstat15minMain[i,"pi0"] <- descstat15minMain[i,"0s"]/descstat15minMain[i,"n"]
  descstat15minMain[i,"rho1"] <- acf(Ii, plot=FALSE)$acf[2]
  descstat15minMain[i,"T"] <- NROW(tmpdata)
  descstat15minMain[i,"S"] <- NCOL(tmpdata)

  rho1st <- rep(NA, NCOL(tmpdata))
  for(j in 1:NCOL(tmpdata)){
    It <- tmpdata[,j]
    rho1st[j] <- acf(It, lag.max=1, plot=FALSE)$acf[2]
  }
  descstat15minMain[i,"maxrho1"] <- max(abs(rho1st), na.rm=TRUE)
  descstat15minMain[i,"minrho1"] <- min(abs(rho1st), na.rm=TRUE)
}


##do tests (main trading hours)
##=============================

##unadjusted tests:
df15minTestsMain <- as.data.frame(NULL)
tmprow <- matrix(NA, 1, 7)
colnames(tmprow) <- c("ticker", "S", "T", "type", "bandwidth",
  "statistic", "pvalue") 
tmprow <- as.data.frame(tmprow)
rownames(tmprow) <- ""

##df-adjusted tests:
df15minTestsMainAdj <- as.data.frame(NULL)
tmprowadj <- matrix(NA, 1, 8)
colnames(tmprowadj) <- c("ticker", "S", "T", "type", "df.adjustment",
  "bandwidth", "statistic", "pvalue") 
tmprowadj <- as.data.frame(tmprowadj)
#rownames(tmprowadj) <- ""

##loop:
for(i in 1:length(list15minIt)){
  
  ##S=all
  ##-----
  
  ##build mIt
  mIt <- list15minIt[[i]]
  t1 <- which(colnames(mIt)=="93000")
  t2 <- which(colnames(mIt)=="160000")
  mIt <- mIt[,t1:t2]
  mIt <- dropvar(mIt)
  
  ##unadjusted LM-test:
  tmp <- doTest(mIt, type="lm", df.adjustment=NULL, hac=TRUE)
  tmprow[1] <- tickers[i]
  for(j in 1:length(tmp)){ tmprow[c(j+1)] <- tmp[[j]] }
  df15minTestsMain <- rbind(df15minTestsMain, tmprow)
      
  ##adjusted LM-test:
  S <- NCOL(mIt) ##for DF-adjustment
  iDFlm <- iDFlmFun(S)
  DFlmAdj <- 1 - iDFlm/NROW(mIt)
  DFlmAdj <- max(0.1, DFlmAdj)
  DFlmAdj
  tmp <- doTest(mIt, type="lm", df.adjustment=DFlmAdj, hac=TRUE)
  tmprowadj[1] <- tickers[i]
  for(j in 1:length(tmp)){ tmprowadj[c(j+1)] <- tmp[[j]] }
  df15minTestsMainAdj <- rbind(df15minTestsMainAdj, tmprowadj)

} #close for(i) loop


##intraday plot of zero-probabilities:
##====================================

windows(5,3)
  
##get current par-values:
def.par <- par(no.readonly=TRUE)

##margins (default=5.1,4.1,4.1,2.1):
par(mar = c(4,3.5,2,1.5)) #distance from: bottom,left,top,right

##make plots:
for(i in 1:length(list1minIt)){
  
  ##intraday zero-probability:
  m15minIt <- list15minIt[[i]]
  pi0t <- 1 - colMeans(m15minIt)
  assetName <- names(list15minIt)[i]
  
  ##create plot:
  mainTxt <- paste0(assetName, ":")
  plot(1:length(pi0t), pi0t, lwd=1, ylim=c(-0.02,1.08), col="white", 
    xaxt="n", #suppress ticks on x-axis
    yaxt="n", #suppress ticks on y-axis
    cex.main=1.5,
    main=mainTxt,
    cex.lab=1.2,                                               
    xlab="",
    ylab="")
  title(line=2,
    xlab=paste0("15-minute interval (S = ", ncol(m15minIt), ")"),
    ylab="Zero-probability")
  lines(as.zoo(pi0t), col="blue", lwd=1)

  ##y-axis:
  abline(h=seq(0,1,by=0.2), col="grey", lty="dotted")
  axis(side=2, at=seq(0,1,by=0.2), padj=0.4,
    labels=c("0.0","0.2","0.4","0.6","0.8","1.0"),
    cex.axis=1.1)
  
  ##x-axis
  xwhere <- seq(8,64,by=8)
  xwhere <- c(1,xwhere)
  axis(side=1, at=xwhere, labels=xwhere, cex.axis=1.1, padj=-0.6)
  whichStart <- which( colnames(m15minIt)=="93000" )
  whichEnd <- which( colnames(m15minIt)=="160000" )
  xwhere <- c(1, whichStart, whichEnd, 64) #4h,9h30,16h,20h
  abline(v=xwhere, col="grey")
  text(c(5,29,49), 1.06, adj=0, col="grey", cex=1,
    labels=c("04:00-09:30", "09:30-16:00", "16:00-20:00"))

  ##redo lines:
  lines(as.zoo(pi0t), col="blue", lwd=1)
  
  ##save plot:
  desttxt <- paste0("./figures/STOCK-15min-", assetName, ".pdf")
#  savePlot(desttxt, type="pdf")

} #close for(i) loop

##return to old par-values:
par(def.par)

##turn off graphics:
graphics.off()


############################################################
## 6 THIRTY-MINUTE RETURNS
############################################################

##create 30-minute timestamps:
whichOnes <- seq(1,length(oneMinStamps),by=30)
thirtyMinStamps <- oneMinStamps[whichOnes]

##create list of 30-min values
##===========================

list30min <- list1min

##fill the list:
for(i in 1:length(tickers)){
  tmpdata <- list1min[[i]]
  tmpdata <- tmpdata[,whichOnes]
  list30min[[i]] <- tmpdata
}

##create list of non-zero indicators
##==================================

list30minIt <- list30min
for(i in 1:length(tickers)){
  tmpdata <- list30min[[i]]
  tmpdataIt <- tmpdata
  for(j in 1:NROW(tmpdata)){
    for(k in 1:NCOL(tmpdata)){
      if( j==1 && k==1 ){
        tmpdataIt[j,k] <- 1
      }else{
        if( k==1 ){
          tmpdataIt[j,k] <-
            ifelse(tmpdata[j,k] == tmpdata[c(j-1),NCOL(tmpdata)], 0, 1)
        }
        if( k>1 ){
          tmpdataIt[j,k] <-
            ifelse(tmpdata[j,k] == tmpdata[j,c(k-1)], 0, 1)
        }
      }
    } #close for(k)
  } #close for(j)
  list30minIt[[i]] <- tmpdataIt
} #close for(i)


##descriptive statistics (all trading hours)
##==========================================

descstat30min <- matrix(NA, length(tickers), 8)
colnames(descstat30min) <-
  c("n", "0s", "pi0", "rho1", "T", "maxrho1", "minrho1", "S")
rownames(descstat30min) <- tickers
for(i in 1:NROW(descstat30min)){
  tmpdata <- list30minIt[[i]]
  Ii <- as.vector(t(tmpdata))

  descstat30min[i,"n"] <- length(Ii)
  descstat30min[i,"0s"] <- length(which(Ii==0))
  descstat30min[i,"pi0"] <- descstat30min[i,"0s"]/descstat30min[i,"n"]
  descstat30min[i,"rho1"] <- acf(Ii, plot=FALSE)$acf[2]
  descstat30min[i,"T"] <- NROW(tmpdata)
  descstat30min[i,"S"] <- NCOL(tmpdata)

  rho1st <- rep(NA, NCOL(tmpdata))
  for(j in 1:NCOL(tmpdata)){
    It <- tmpdata[,j]
    rho1st[j] <- acf(It, lag.max=1, plot=FALSE)$acf[2]
  }
  descstat30min[i,"maxrho1"] <- max(abs(rho1st), na.rm=TRUE)
  descstat30min[i,"minrho1"] <- min(abs(rho1st), na.rm=TRUE)
}


##do tests (all trading hours)
##============================

##unadjusted tests:
df30minTests <- as.data.frame(NULL)
tmprow <- matrix(NA, 1, 7)
colnames(tmprow) <- c("ticker", "S", "T", "type", "bandwidth",
  "statistic", "pvalue") 
tmprow <- as.data.frame(tmprow)

##df-adjusted tests:
df30minTestsAdj <- as.data.frame(NULL)
tmprowadj <- matrix(NA, 1, 8)
colnames(tmprowadj) <- c("ticker", "S", "T", "type", "df.adjustment",
  "bandwidth", "statistic", "pvalue") 
tmprowadj <- as.data.frame(tmprowadj)

##loop:
for(i in 1:length(list30minIt)){
  
  ##S=48
  ##----
  
  ##build mIt
  mIt <- list30minIt[[i]]
  mIt <- dropvar(mIt)
    
  ##unadjusted LM-test:
  tmp <- doTest(mIt, type="lm", df.adjustment=NULL, hac=TRUE)
  tmprow[1] <- tickers[i]
  for(j in 1:length(tmp)){ tmprow[c(j+1)] <- tmp[[j]] }
  df30minTests <- rbind(df30minTests, tmprow)
      
  ##adjusted LM-test:
  S <- NCOL(mIt) ##for DF-adjustment
  iDFlm <- iDFlmFun(S)
  DFlmAdj <- 1 - iDFlm/NROW(mIt)
  DFlmAdj <- max(0.1, DFlmAdj)
  DFlmAdj
  tmp <- doTest(mIt, type="lm", df.adjustment=DFlmAdj, hac=TRUE)
  tmprowadj[1] <- tickers[i]
  for(j in 1:length(tmp)){ tmprowadj[c(j+1)] <- tmp[[j]] }
  df30minTestsAdj <- rbind(df30minTestsAdj, tmprowadj)

} #close for(i) loop


##descriptive statistics (main trading hours)
##===========================================

descstat30minMain <- matrix(NA, length(tickers), 8)
colnames(descstat30minMain) <-
  c("n", "0s", "pi0", "rho1", "T", "maxrho1", "minrho1", "S")
rownames(descstat30minMain) <- tickers
for(i in 1:NROW(descstat30minMain)){
  tmpdata <- list30minIt[[i]]
  t1 <- which(colnames(tmpdata)=="93000")
  t2 <- which(colnames(tmpdata)=="160000")
  tmpdata <- tmpdata[,t1:t2]
  Ii <- as.vector(t(tmpdata))

  descstat30minMain[i,"n"] <- length(Ii)
  descstat30minMain[i,"0s"] <- length(which(Ii==0))
  descstat30minMain[i,"pi0"] <- descstat30minMain[i,"0s"]/descstat30minMain[i,"n"]
  descstat30minMain[i,"rho1"] <- acf(Ii, plot=FALSE)$acf[2]
  descstat30minMain[i,"T"] <- NROW(tmpdata)
  descstat30minMain[i,"S"] <- NCOL(tmpdata)

  rho1st <- rep(NA, NCOL(tmpdata))
  for(j in 1:NCOL(tmpdata)){
    It <- tmpdata[,j]
    rho1st[j] <- acf(It, lag.max=1, plot=FALSE)$acf[2]
  }
  descstat30minMain[i,"maxrho1"] <- max(abs(rho1st), na.rm=TRUE)
  descstat30minMain[i,"minrho1"] <- min(abs(rho1st), na.rm=TRUE)
}


##do tests (main trading hours)
##=============================

##unadjusted tests:
df30minTestsMain <- as.data.frame(NULL)
tmprow <- matrix(NA, 1, 7)
colnames(tmprow) <- c("ticker", "S", "T", "type", "bandwidth",
  "statistic", "pvalue") 
tmprow <- as.data.frame(tmprow)
#rownames(tmprow) <- ""

##df-adjusted tests:
df30minTestsMainAdj <- as.data.frame(NULL)
tmprowadj <- matrix(NA, 1, 8)
colnames(tmprowadj) <- c("ticker", "S", "T", "type", "df.adjustment",
  "bandwidth", "statistic", "pvalue") 
tmprowadj <- as.data.frame(tmprowadj)
#rownames(tmprowadj) <- ""

##loop:
for(i in 1:length(list30minIt)){
  
  ##S=48
  ##----
  
  ##build mIt
  mIt <- list30minIt[[i]]
  t1 <- which(colnames(mIt)=="93000")
  t2 <- which(colnames(mIt)=="160000")
  mIt <- mIt[,t1:t2]
  mIt <- dropvar(mIt)
    
  ##unadjusted LM-test:
  tmp <- doTest(mIt, type="lm", df.adjustment=NULL, hac=TRUE)
  tmprow[1] <- tickers[i]
  for(j in 1:length(tmp)){ tmprow[c(j+1)] <- tmp[[j]] }
  df30minTestsMain <- rbind(df30minTestsMain, tmprow)
      
  ##adjusted LM-test:
  S <- NCOL(mIt) ##for DF-adjustment
  iDFlm <- iDFlmFun(S)
  DFlmAdj <- 1 - iDFlm/NROW(mIt)
  DFlmAdj <- max(0.1, DFlmAdj)
  DFlmAdj
  tmp <- doTest(mIt, type="lm", df.adjustment=DFlmAdj, hac=TRUE)
  tmprowadj[1] <- tickers[i]
  for(j in 1:length(tmp)){ tmprowadj[c(j+1)] <- tmp[[j]] }
  df30minTestsMainAdj <- rbind(df30minTestsMainAdj, tmprowadj)

} #close for(i) loop


##intraday plot of zero-probabilities:
##====================================

windows(5,3)
  
##get current par-values:
def.par <- par(no.readonly=TRUE)

##margins (default=5.1,4.1,4.1,2.1):
par(mar = c(4,3.5,2,1.5)) #distance from: bottom,left,top,right

##make plots:
for(i in 1:length(list1minIt)){
  
  ##intraday zero-probability:
  m30minIt <- list30minIt[[i]]
  pi0t <- 1 - colMeans(m30minIt)
  assetName <- names(list30minIt)[i]
  
  ##create plot:
  mainTxt <- paste0(assetName, ":")
  plot(1:length(pi0t), pi0t, lwd=1, ylim=c(-0.02,1.08), col="white", 
    xaxt="n", #suppress ticks on x-axis
    yaxt="n", #suppress ticks on y-axis
    cex.main=1.5,
    main=mainTxt,
    cex.lab=1.2,
    xlab="",
    ylab="")
  title(line=2,
    xlab=paste0("30-minute interval (S = ", ncol(m30minIt), ")"),
    ylab="Zero-probability")
  lines(as.zoo(pi0t), col="blue", lwd=1)
  
  ##y-axis:
  abline(h=seq(0,1,by=0.2), col="grey", lty="dotted")
  axis(side=2, at=seq(0,1,by=0.2), padj=0.4,
    labels=c("0.0","0.2","0.4","0.6","0.8","1.0"),
    cex.axis=1.1)
  
  ##x-axis
  xwhere <- seq(4,32,by=4)
  xwhere <- c(1,xwhere)
  axis(side=1, at=xwhere, labels=xwhere, cex.axis=1.1, padj=-0.6)
  whichStart <- which( colnames(m30minIt)=="93000" )
  whichEnd <- which( colnames(m30minIt)=="160000" )
  xwhere <- c(1, whichStart, whichEnd, 32) #4h,9h30,16h,20h
  abline(v=xwhere, col="grey")
  text(c(3,15,26), 1.06, adj=0, col="grey", cex=1,
    labels=c("04:00-09:30", "09:30-16:00", "16h-20h"))
  
  ##redo lines:
  lines(as.zoo(pi0t), col="blue", lwd=1)
  
  ##save plot:
  desttxt <- paste0("./figures/STOCK-30min-", assetName, ".pdf")
#  savePlot(desttxt, type="pdf")

} #close for(i) loop

##return to old par-values:
par(def.par)

##turn off graphics:
graphics.off()


############################################################
## 7 HOURLY RETURNS
############################################################

##create 60-minute timestamps:
whichOnes <- seq(1,length(oneMinStamps),by=60)
sixtyMinStamps <- oneMinStamps[whichOnes]

##create list of 60-min values
##============================

list60min <- list1min

##fill the list:
for(i in 1:length(tickers)){
  tmpdata <- list1min[[i]]
  tmpdata <- tmpdata[,whichOnes]
  list60min[[i]] <- tmpdata
}

##create list of non-zero indicators
##==================================

list60minIt <- list60min
for(i in 1:length(tickers)){
  tmpdata <- list60min[[i]]
  tmpdataIt <- tmpdata
  for(j in 1:NROW(tmpdata)){
    for(k in 1:NCOL(tmpdata)){
      if( j==1 && k==1 ){
        tmpdataIt[j,k] <- 1
      }else{
        if( k==1 ){
          tmpdataIt[j,k] <-
            ifelse(tmpdata[j,k] == tmpdata[c(j-1),NCOL(tmpdata)], 0, 1)
        }
        if( k>1 ){
          tmpdataIt[j,k] <-
            ifelse(tmpdata[j,k] == tmpdata[j,c(k-1)], 0, 1)
        }
      }
    } #close for(k)
  } #close for(j)
  list60minIt[[i]] <- tmpdataIt
} #close for(i)


##descriptive statistics (all trading hours)
##==========================================

descstat60min <- matrix(NA, length(tickers), 8)
colnames(descstat60min) <-
  c("n", "0s", "pi0", "rho1", "T", "maxrho1", "minrho1", "S")
rownames(descstat60min) <- tickers
for(i in 1:NROW(descstat60min)){
  tmpdata <- list60minIt[[i]]
  Ii <- as.vector(t(tmpdata))

  descstat60min[i,"n"] <- length(Ii)
  descstat60min[i,"0s"] <- length(which(Ii==0))
  descstat60min[i,"pi0"] <- descstat60min[i,"0s"]/descstat60min[i,"n"]
  descstat60min[i,"rho1"] <- acf(Ii, plot=FALSE)$acf[2]
  descstat60min[i,"T"] <- NROW(tmpdata)
  descstat60min[i,"S"] <- NCOL(tmpdata)

  rho1st <- rep(NA, NCOL(tmpdata))
  for(j in 1:NCOL(tmpdata)){
    It <- tmpdata[,j]
    rho1st[j] <- acf(It, lag.max=1, plot=FALSE)$acf[2]
  }
  descstat60min[i,"maxrho1"] <- max(abs(rho1st), na.rm=TRUE)
  descstat60min[i,"minrho1"] <- min(abs(rho1st), na.rm=TRUE)
}


##do tests (all trading hours)
##============================

##unadjusted tests:
df60minTests <- as.data.frame(NULL)
tmprow <- matrix(NA, 1, 7)
colnames(tmprow) <- c("ticker", "S", "T", "type", "bandwidth",
  "statistic", "pvalue") 
tmprow <- as.data.frame(tmprow)

##df-adjusted tests:
df60minTestsAdj <- as.data.frame(NULL)
tmprowadj <- matrix(NA, 1, 8)
colnames(tmprowadj) <- c("ticker", "S", "T", "type", "df.adjustment",
  "bandwidth", "statistic", "pvalue") 
tmprowadj <- as.data.frame(tmprowadj)

##loop:
for(i in 1:length(list60minIt)){
  
  ##S=all
  ##-----
  
  ##build mIt
  mIt <- list60minIt[[i]]
  mIt <- dropvar(mIt)
    
  ##unadjusted LM-test:
  tmp <- doTest(mIt, type="lm", df.adjustment=NULL, hac=TRUE)
  tmprow[1] <- tickers[i]
  for(j in 1:length(tmp)){ tmprow[c(j+1)] <- tmp[[j]] }
  df60minTests <- rbind(df60minTests, tmprow)
      
  ##adjusted LM-test:
  S <- NCOL(mIt) ##for DF-adjustment
  iDFlm <- iDFlmFun(S)
  DFlmAdj <- 1 - iDFlm/NROW(mIt)
  DFlmAdj <- max(0.1, DFlmAdj)
  DFlmAdj
  tmp <- doTest(mIt, type="lm", df.adjustment=DFlmAdj, hac=TRUE)
  tmprowadj[1] <- tickers[i]
  for(j in 1:length(tmp)){ tmprowadj[c(j+1)] <- tmp[[j]] }
  df60minTestsAdj <- rbind(df60minTestsAdj, tmprowadj)

} #close for(i) loop


##descriptive statistics (main trading hours)
##===========================================

descstat60minMain <- matrix(NA, length(tickers), 8)
colnames(descstat60minMain) <-
  c("n", "0s", "pi0", "rho1", "T", "maxrho1", "minrho1", "S")
rownames(descstat60minMain) <- tickers
for(i in 1:NROW(descstat60minMain)){
  tmpdata <- list60minIt[[i]]
  t1 <- which(colnames(tmpdata)=="100000")
  t2 <- which(colnames(tmpdata)=="160000")
  tmpdata <- tmpdata[,t1:t2]
  Ii <- as.vector(t(tmpdata))

  descstat60minMain[i,"n"] <- length(Ii)
  descstat60minMain[i,"0s"] <- length(which(Ii==0))
  descstat60minMain[i,"pi0"] <- descstat60minMain[i,"0s"]/descstat60minMain[i,"n"]
  descstat60minMain[i,"rho1"] <- acf(Ii, plot=FALSE)$acf[2]
  descstat60minMain[i,"T"] <- NROW(tmpdata)
  descstat60minMain[i,"S"] <- NCOL(tmpdata)

  rho1st <- rep(NA, NCOL(tmpdata))
  for(j in 1:NCOL(tmpdata)){
    It <- tmpdata[,j]
    rho1st[j] <- acf(It, lag.max=1, plot=FALSE)$acf[2]
  }
  descstat60minMain[i,"maxrho1"] <- max(abs(rho1st), na.rm=TRUE)
  descstat60minMain[i,"minrho1"] <- min(abs(rho1st), na.rm=TRUE)
}


##do tests (main trading hours)
##============================

##unadjusted tests:
df60minTestsMain <- as.data.frame(NULL)
tmprow <- matrix(NA, 1, 7)
colnames(tmprow) <- c("ticker", "S", "T", "type", "bandwidth",
  "statistic", "pvalue") 
tmprow <- as.data.frame(tmprow)

##df-adjusted tests:
df60minTestsMainAdj <- as.data.frame(NULL)
tmprowadj <- matrix(NA, 1, 8)
colnames(tmprowadj) <- c("ticker", "S", "T", "type", "df.adjustment",
  "bandwidth", "statistic", "pvalue") 
tmprowadj <- as.data.frame(tmprowadj)

##loop:
for(i in 1:length(list60minIt)){
  
  ##S=all
  ##-----
  
  ##build mIt
  mIt <- list60minIt[[i]]
  t1 <- which(colnames(mIt)=="100000")
  t2 <- which(colnames(mIt)=="160000")
  mIt <- mIt[,t1:t2]
  mIt <- dropvar(mIt)
    
  ##unadjusted LM-test:
  tmp <- doTest(mIt, type="lm", df.adjustment=NULL, hac=TRUE)
  tmprow[1] <- tickers[i]
  for(j in 1:length(tmp)){ tmprow[c(j+1)] <- tmp[[j]] }
  df60minTestsMain <- rbind(df60minTestsMain, tmprow)
      
  ##adjusted LM-test:
  S <- NCOL(mIt) ##for DF-adjustment
  iDFlm <- iDFlmFun(S)
  DFlmAdj <- 1 - iDFlm/NROW(mIt)
  DFlmAdj <- max(0.1, DFlmAdj)
  DFlmAdj
  tmp <- doTest(mIt, type="lm", df.adjustment=DFlmAdj, hac=TRUE)
  tmprowadj[1] <- tickers[i]
  for(j in 1:length(tmp)){ tmprowadj[c(j+1)] <- tmp[[j]] }
  df60minTestsMainAdj <- rbind(df60minTestsMainAdj, tmprowadj)

} #close for(i) loop


##intraday plot of zero-probabilities:
##====================================

windows(5,3)
  
##get current par-values:
def.par <- par(no.readonly=TRUE)

##margins (default=5.1,4.1,4.1,2.1):
par(mar = c(4,3.5,2,1.5)) #distance from: bottom,left,top,right

##make plots:
for(i in 1:length(list1minIt)){
  
  ##intraday zero-probability:
  m60minIt <- list60minIt[[i]]
  pi0t <- 1 - colMeans(m60minIt)
  assetName <- names(list60minIt)[i]
  S <- length(pi0t)
    
  ##create plot:
  mainTxt <- paste0(assetName, ":")
  plot(1:length(pi0t), pi0t, lwd=1, ylim=c(-0.02,1.08), col="white", 
    xaxt="n", #suppress ticks on x-axis
    yaxt="n", #suppress ticks on y-axis
    cex.main=1.5,
    main=mainTxt,
    cex.lab=1.2,
    xlab="",
    ylab="")
  title(line=2,
    xlab=paste0("60-minute interval (S = ", S, ")"),
    ylab="Zero-probability")
  lines(as.zoo(pi0t), col="blue", lwd=1)
  
  ##y-axis:
  abline(h=seq(0,1,by=0.2), col="grey", lty="dotted")
  axis(side=2, at=seq(0,1,by=0.2), padj=0.4,
    labels=c("0.0","0.2","0.4","0.6","0.8","1.0"),
    cex.axis=1.1)
  
  ##x-axis
  xwhere <- seq(2,20,by=2)
  axis(side=1, at=xwhere, labels=xwhere, cex.axis=1.1, padj=-0.6)
  whichStart <- 0.5 + which( colnames(m60minIt)=="90000" )
  whichEnd <- which( colnames(m60minIt)=="160000" )
  xwhere <- c(1, whichStart, whichEnd, 16) #4h,9h30,16h,20h
  abline(v=xwhere, col="grey")
  text(c(2,8,13.2), 1.06, adj=0, col="grey", cex=1,
    labels=c("04:00-09:30", "09:30-16:00", "16h-20h"))
  
  ##redo lines:
  lines(as.zoo(pi0t), col="blue", lwd=1)
  
  ##save plot:
  desttxt <- paste0("./figures/STOCK-60min-", assetName, ".pdf")
#  savePlot(desttxt, type="pdf")

} #close for(i) loop

##return to old par-values:
par(def.par)

##turn off graphics:
graphics.off()


###################################################
## 8 PRINT RESULTS FOR PAPER
###################################################

##table w/descriptive statistics (all trading hours)
##==================================================

tmp <- matrix(NA, 5, 8)
colnames(tmp) <- c("n", "0s", "pi0", "rho1", "T", "maxrho1",
  "minrho1", "S")
rownames(tmp) <-
  c("Hourly", "30-minute", "15-minute", "5-minute", "1-minute")
for(i in tickers){
  tmp[1,] <- descstat60min[i,]
  tmp[2,] <- descstat30min[i,]
  tmp[3,] <- descstat15min[i,]
  tmp[4,] <- descstat5min[i,]
  tmp[5,] <- descstat1min[i,]
  print(paste0(i, ":"))

  ##print LaTeX code:  
  tmpTex <- xtable(tmp[,-7],
    align=c("l","r","r","r","r","r","r","r"),
    digits=c(0,0,0,3,3,0,3,0))
  print(tmpTex,
      floating=FALSE, ##FALSE: removes table environment
      math.style.negative=TRUE, ##use $-$ for negative sign
      NA.string="--",
      )     

}


##table with tests (all trading hours)
##====================================

##create matrix:
tmp <- matrix(NA, 8, 6)
colnames(tmp) <- c("lmstat1", "pvalue1",
  "lmstat2", "pvalue2", "S", "T")
rownames(tmp) <- c("Hourly", "30-minute", "15-minute",
  "5-minute(*)", "5-minute", "1-minute(**)", "1-minute(*)", "1-minute")
tmp <- as.data.frame(tmp)

##loop over tickers:
for(i in tickers){

  ##60-minute frequency:
  whichOnes <- which( i == df60minTests[,"ticker"] )
  tmp["Hourly", "lmstat1"] <- df60minTests[whichOnes, "statistic"]
  tmp["Hourly", "pvalue1"] <- df60minTests[whichOnes, "pvalue"]
  tmp["Hourly", "lmstat2"] <- df60minTestsAdj[whichOnes, "statistic"]
  tmp["Hourly", "pvalue2"] <- df60minTestsAdj[whichOnes, "pvalue"]
  tmp["Hourly", "S"] <- df60minTests[whichOnes, "S"]
  tmp["Hourly", "T"] <- df60minTests[whichOnes, "T"]
  
  ##30-minute frequency:
  whichOnes <- which( i == df30minTests[,"ticker"] )
  tmp["30-minute", "lmstat1"] <- df30minTests[whichOnes, "statistic"]
  tmp["30-minute", "pvalue1"] <- df30minTests[whichOnes, "pvalue"]
  tmp["30-minute", "lmstat2"] <- df30minTestsAdj[whichOnes, "statistic"]
  tmp["30-minute", "pvalue2"] <- df30minTestsAdj[whichOnes, "pvalue"]
  tmp["30-minute", "S"] <- df30minTests[whichOnes, "S"]
  tmp["30-minute", "T"] <- df30minTests[whichOnes, "T"]
  
  ##15-minute frequency:
  whichOnes <- which( i == df15minTests[,"ticker"] )
  tmp["15-minute", "lmstat1"] <- df15minTests[whichOnes, "statistic"]
  tmp["15-minute", "pvalue1"] <- df15minTests[whichOnes, "pvalue"]
  tmp["15-minute", "lmstat2"] <- df15minTestsAdj[whichOnes, "statistic"]
  tmp["15-minute", "pvalue2"] <- df15minTestsAdj[whichOnes, "pvalue"]
  tmp["15-minute", "S"] <- df15minTests[whichOnes, "S"]
  tmp["15-minute", "T"] <- df15minTests[whichOnes, "T"]
  
  ##5-minute(*) frequency (i.e. every third intraday period):
  whichOnes <- which( i == df5minTests[,"ticker"] )
  tmp["5-minute(*)", "lmstat1"] <- df5minTests[whichOnes[2], "statistic"]
  tmp["5-minute(*)", "pvalue1"] <- df5minTests[whichOnes[2], "pvalue"]
  tmp["5-minute(*)", "lmstat2"] <- df5minTestsAdj[whichOnes[2], "statistic"]
  tmp["5-minute(*)", "pvalue2"] <- df5minTestsAdj[whichOnes[2], "pvalue"]
  tmp["5-minute(*)", "S"] <- df5minTests[whichOnes[2], "S"]
  tmp["5-minute(*)", "T"] <- df5minTests[whichOnes[2], "T"]
  
  ##5-minute frequency (i.e. all intraday periods):
  whichOnes <- which( i == df5minTests[,"ticker"] )
  tmp["5-minute", "lmstat1"] <- df5minTests[whichOnes[1], "statistic"]
  tmp["5-minute", "pvalue1"] <- df5minTests[whichOnes[1], "pvalue"]
  tmp["5-minute", "lmstat2"] <- df5minTestsAdj[whichOnes[1], "statistic"]
  tmp["5-minute", "pvalue2"] <- df5minTestsAdj[whichOnes[1], "pvalue"]
  tmp["5-minute", "S"] <- df5minTests[whichOnes[1], "S"]
  tmp["5-minute", "T"] <- df5minTests[whichOnes[1], "T"]

  ##1-minute(**) frequency (i.e. every third intraday period):
  whichOnes <- which( i == df1minTests[,"ticker"] )
  tmp["1-minute(**)", "lmstat1"] <- df1minTests[whichOnes[3], "statistic"]
  tmp["1-minute(**)", "pvalue1"] <- df1minTests[whichOnes[3], "pvalue"]
  tmp["1-minute(**)", "lmstat2"] <- df1minTestsAdj[whichOnes[3], "statistic"]
  tmp["1-minute(**)", "pvalue2"] <- df1minTestsAdj[whichOnes[3], "pvalue"]
  tmp["1-minute(**)", "S"] <- df1minTests[whichOnes[3], "S"]
  tmp["1-minute(**)", "T"] <- df1minTests[whichOnes[3], "T"]

  ##1-minute(*) frequency (i.e. every third intraday period):
  whichOnes <- which( i == df1minTests[,"ticker"] )
  tmp["1-minute(*)", "lmstat1"] <- df1minTests[whichOnes[2], "statistic"]
  tmp["1-minute(*)", "pvalue1"] <- df1minTests[whichOnes[2], "pvalue"]
  tmp["1-minute(*)", "lmstat2"] <- df1minTestsAdj[whichOnes[2], "statistic"]
  tmp["1-minute(*)", "pvalue2"] <- df1minTestsAdj[whichOnes[2], "pvalue"]
  tmp["1-minute(*)", "S"] <- df1minTests[whichOnes[2], "S"]
  tmp["1-minute(*)", "T"] <- df1minTests[whichOnes[2], "T"]

  ##1-minute(*) frequency (i.e. every third intraday period):
  whichOnes <- which( i == df1minTests[,"ticker"] )
  tmp["1-minute", "lmstat1"] <- df1minTests[whichOnes[1], "statistic"]
  tmp["1-minute", "pvalue1"] <- df1minTests[whichOnes[1], "pvalue"]
  tmp["1-minute", "lmstat2"] <- df1minTestsAdj[whichOnes[1], "statistic"]
  tmp["1-minute", "pvalue2"] <- df1minTestsAdj[whichOnes[1], "pvalue"]
  tmp["1-minute", "S"] <- df1minTests[whichOnes[1], "S"]
  tmp["1-minute", "T"] <- df1minTests[whichOnes[1], "T"]
  
  ##print LaTeX code:  
  tmpTex <- xtable(tmp,
    align=c("l","r","r","r","r","r","r"),
    digits=c(0,1,4,1,4,0,0))
  print(paste0(i, ": "))
  print(tmpTex,
      floating=FALSE, ##FALSE: removes table environment
      math.style.negative=TRUE, ##use $-$ for negative sign
      NA.string="--",
      )     

} #close for(i)


##table w/descriptive statistics (main trading hours)
##===================================================

tmp <- matrix(NA, 5, 8)
colnames(tmp) <- c("n", "0s", "pi0", "rho1", "T", "maxrho1",
  "minrho1", "S")
rownames(tmp) <-
  c("Hourly", "30-minute", "15-minute", "5-minute", "1-minute")
for(i in tickers){
  tmp[1,] <- descstat60minMain[i,]
  tmp[2,] <- descstat30minMain[i,]
  tmp[3,] <- descstat15minMain[i,]
  tmp[4,] <- descstat5minMain[i,]
  tmp[5,] <- descstat1minMain[i,]
  print(paste0(i, ":"))

  ##print LaTeX code:  
  tmpTex <- xtable(tmp[,-7],
    align=c("l","r","r","r","r","r","r","r"),
    digits=c(0,0,0,3,3,0,3,0))
  print(tmpTex,
      floating=FALSE, ##FALSE: removes table environment
      math.style.negative=TRUE, ##use $-$ for negative sign
      NA.string="--",
      )     

}


##table with tests (main trading hours)
##=====================================

##create matrix:
tmp <- matrix(NA, 8, 6)
colnames(tmp) <- c("lmstat1", "pvalue1",
  "lmstat2", "pvalue2", "S", "T")
rownames(tmp) <- c("Hourly", "30-minute", "15-minute",
  "5-minute(*)", "5-minute", "1-minute(**)", "1-minute(*)", "1-minute")
tmp <- as.data.frame(tmp)

##loop over tickers:
for(i in tickers){

  ##60-minute frequency:
  whichOnes <- which( i == df60minTestsMain[,"ticker"] )
  tmp["Hourly", "lmstat1"] <- df60minTestsMain[whichOnes, "statistic"]
  tmp["Hourly", "pvalue1"] <- df60minTestsMain[whichOnes, "pvalue"]
  tmp["Hourly", "lmstat2"] <- df60minTestsMainAdj[whichOnes, "statistic"]
  tmp["Hourly", "pvalue2"] <- df60minTestsMainAdj[whichOnes, "pvalue"]
  tmp["Hourly", "S"] <- df60minTestsMain[whichOnes, "S"]
  tmp["Hourly", "T"] <- df60minTestsMain[whichOnes, "T"]
  
  ##30-minute frequency:
  whichOnes <- which( i == df30minTestsMain[,"ticker"] )
  tmp["30-minute", "lmstat1"] <- df30minTestsMain[whichOnes, "statistic"]
  tmp["30-minute", "pvalue1"] <- df30minTestsMain[whichOnes, "pvalue"]
  tmp["30-minute", "lmstat2"] <- df30minTestsMainAdj[whichOnes, "statistic"]
  tmp["30-minute", "pvalue2"] <- df30minTestsMainAdj[whichOnes, "pvalue"]
  tmp["30-minute", "S"] <- df30minTestsMain[whichOnes, "S"]
  tmp["30-minute", "T"] <- df30minTestsMain[whichOnes, "T"]
  
  ##15-minute frequency:
  whichOnes <- which( i == df15minTestsMain[,"ticker"] )
  tmp["15-minute", "lmstat1"] <- df15minTestsMain[whichOnes, "statistic"]
  tmp["15-minute", "pvalue1"] <- df15minTestsMain[whichOnes, "pvalue"]
  tmp["15-minute", "lmstat2"] <- df15minTestsMainAdj[whichOnes, "statistic"]
  tmp["15-minute", "pvalue2"] <- df15minTestsMainAdj[whichOnes, "pvalue"]
  tmp["15-minute", "S"] <- df15minTestsMain[whichOnes, "S"]
  tmp["15-minute", "T"] <- df15minTestsMain[whichOnes, "T"]
  
  ##5-minute(*) frequency (i.e. every third intraday period):
  whichOnes <- which( i == df5minTestsMain[,"ticker"] )
  tmp["5-minute(*)", "lmstat1"] <- df5minTestsMain[whichOnes[2], "statistic"]
  tmp["5-minute(*)", "pvalue1"] <- df5minTestsMain[whichOnes[2], "pvalue"]
  tmp["5-minute(*)", "lmstat2"] <- df5minTestsMainAdj[whichOnes[2], "statistic"]
  tmp["5-minute(*)", "pvalue2"] <- df5minTestsMainAdj[whichOnes[2], "pvalue"]
  tmp["5-minute(*)", "S"] <- df5minTestsMain[whichOnes[2], "S"]
  tmp["5-minute(*)", "T"] <- df5minTestsMain[whichOnes[2], "T"]
  
  ##5-minute frequency (i.e. all intraday periods):
  whichOnes <- which( i == df5minTestsMain[,"ticker"] )
  tmp["5-minute", "lmstat1"] <- df5minTestsMain[whichOnes[1], "statistic"]
  tmp["5-minute", "pvalue1"] <- df5minTestsMain[whichOnes[1], "pvalue"]
  tmp["5-minute", "lmstat2"] <- df5minTestsMainAdj[whichOnes[1], "statistic"]
  tmp["5-minute", "pvalue2"] <- df5minTestsMainAdj[whichOnes[1], "pvalue"]
  tmp["5-minute", "S"] <- df5minTestsMain[whichOnes[1], "S"]
  tmp["5-minute", "T"] <- df5minTestsMain[whichOnes[1], "T"]

  ##1-minute(**) frequency (every sixth intraday period):
  whichOnes <- which( i == df1minTestsMain[,"ticker"] )
  tmp["1-minute(**)", "lmstat1"] <- df1minTestsMain[whichOnes[3], "statistic"]
  tmp["1-minute(**)", "pvalue1"] <- df1minTestsMain[whichOnes[3], "pvalue"]
  tmp["1-minute(**)", "lmstat2"] <- df1minTestsMainAdj[whichOnes[3], "statistic"]
  tmp["1-minute(**)", "pvalue2"] <- df1minTestsMainAdj[whichOnes[3], "pvalue"]
  tmp["1-minute(**)", "S"] <- df1minTestsMain[whichOnes[3], "S"]
  tmp["1-minute(**)", "T"] <- df1minTestsMain[whichOnes[3], "T"]

  ##1-minute(*) frequency (every third intraday period):
  whichOnes <- which( i == df1minTestsMain[,"ticker"] )
  tmp["1-minute(*)", "lmstat1"] <- df1minTestsMain[whichOnes[2], "statistic"]
  tmp["1-minute(*)", "pvalue1"] <- df1minTestsMain[whichOnes[2], "pvalue"]
  tmp["1-minute(*)", "lmstat2"] <- df1minTestsMainAdj[whichOnes[2], "statistic"]
  tmp["1-minute(*)", "pvalue2"] <- df1minTestsMainAdj[whichOnes[2], "pvalue"]
  tmp["1-minute(*)", "S"] <- df1minTestsMain[whichOnes[2], "S"]
  tmp["1-minute(*)", "T"] <- df1minTestsMain[whichOnes[2], "T"]

  ##1-minute frequency (all periods):
  whichOnes <- which( i == df1minTestsMain[,"ticker"] )
  tmp["1-minute", "lmstat1"] <- df1minTestsMain[whichOnes[1], "statistic"]
  tmp["1-minute", "pvalue1"] <- df1minTestsMain[whichOnes[1], "pvalue"]
  tmp["1-minute", "lmstat2"] <- df1minTestsMainAdj[whichOnes[1], "statistic"]
  tmp["1-minute", "pvalue2"] <- df1minTestsMainAdj[whichOnes[1], "pvalue"]
  tmp["1-minute", "S"] <- df1minTestsMain[whichOnes[1], "S"]
  tmp["1-minute", "T"] <- df1minTestsMain[whichOnes[1], "T"]

  ##print LaTeX code:  
  tmpTex <- xtable(tmp,
    align=c("l","r","r","r","r","r","r"),
    digits=c(0,1,4,1,4,0,0))
  print(paste0(i, ": "))
  print(tmpTex,
      floating=FALSE, ##FALSE: removes table environment
      math.style.negative=TRUE, ##use $-$ for negative sign
      NA.string="--",
      )     

} #close for(i)

