###########################################################
## This is an R file part of the project:
##
##   "Testing the zero-process of intraday financial
##    returns for non-stationary periodicity"
##
## by Ovidijus Stauskas and Genaro Sucarrat
##
## This R file undertakes Monte Carlo simulations of the
## tests.
##
## First created in February 2023, Oslo.
##
## Contents:
##
## 1 INITIATE
## 2 AUXILIARY FUNCTIONS
## 3 SIMULATION PARAMETERS
## 4 SIMULATE
## 5 RESULTS
##
###########################################################


###########################################################
##1 INITIATE
###########################################################

##clean workspace:
rm(list = ls())                                                  

##load required CRAN packages:
library(psych)
library(BinNor)
library(gets)


###########################################################
## 2 AUXILIARY FUNCTIONS
###########################################################

## This function takes a matrix x and prints a very basic
## and simple representation intended for tabular
## objects in LaTeX

totex <- function(x, digits=3){
  for(i in 1:NROW(x)){
    tmp <- rownames(x)[i]
    result <- tmp
    for(j in 1:NCOL(x)){
      tmp <- format(round(x[i,j], digits=digits), nsmall=digits)  
      tmp <- paste0("& ", tmp)
      result <- c(result, tmp)
    }
    cat(result, "\\\\ \n")
  }    
}

## This function simulates from a multivariate binary
## distribution:
##
##  n:            no. of observations
##  probs:        vector of marginal probabilities of a 1
##  chk.probs:    logical
##  corrmat:      correlation matrix
##                contemporaneous correlations of the It series
##  chk.corrmat:  logical
##
## value returned by function: an (n x S) matrix with
## the I_st's.

rmbinary <- function(n, probs=NULL, chk.probs=TRUE, corrmat=NULL,
  chk.corrmat=TRUE)
{
  ##compute sigma_star:
  ##===================
  
  no.bin <- length(probs)
  p <- probs
  q <- 1 - p
  sigma_star <- diag(no.bin)
  for (i in 1:no.bin) {
    for (j in 1:no.bin) {
      if (i > j ){ 
        sigma_star[i, j] <- psych:::phi2tet(corrmat[i, j], p[i], p[j],
          n=NULL, correct=TRUE)
      }
    }
  }
  
  ##simulate from multivariate normal:
  ##==================================

  means <- rep(0, no.bin)
  ev <- eigen(sigma_star, symmetric = TRUE) #only lower triangular of sigma_star is used
  R <- t(ev$vectors %*% ( t(ev$vectors) * sqrt(pmax(ev$values,0)) ) )
  result <- matrix(rnorm(n * NCOL(sigma_star)), nrow = n, byrow = TRUE) %*% R
  result <- sweep(result, 2, means, "+")

  ##convert to multivariate binary:
  ##===============================

  for (i in 1:n) {
    for (j in 1:no.bin) {
        if (result[i, j] <= qnorm(1 - p[j])) 
          result[i, j] <- 0
        else result[i, j] <- 1
    }
  }
  
  ##result:
  ##=======
  
  return(result)

} #close rmbinary()


## This functions simulates the It's in DGP D. It relies
## on the CRAN package "BinNor". Function arguments:
##
##  n:          no. of observations
##  burn.in:    no. of burn-in observations
##  S:          no. of intraday periods
##  cmat:       correlation matrix
##              contemporaneous correlations of the It series
##  intercept   vector of intercept coefficients of a dynamic logit
##  ar1coef     the ar(1) coefficient in a dynamic logit
##
## value returned by function: an (n x S) matrix with
## the I_st's.

dgpdSim <- function(n, burn.in=20, S=2, cmat=NULL, intercept=rep(1.4,S),
  ar1coef=0.9)
{
  ##check S vs. length(intercept)
  ##=====================================
  
  if( S != length(intercept) ){
    stop("S must equal length(intercept)")
  }

  ##create results matrices:
  ##========================
  
  mIt <- matrix(NA, n+burn.in, S)
  mIt[1,] <- 1 #initial value
  mht <- matrix(NA, n+burn.in, S)
  mht[1,] <- 0.5 #initial value
  mPIt <- matrix(NA, n+burn.in, S)
  mPIt[1,] <- 1/(1+exp(-mht[1,]))

  ##loop:
  ##=====
  
  ##loop over i:
  for(i in 2:c(n+burn.in)){
    
    ##loop over S:
    imin1 <- c(i-1)
    for(j in 1:S){
      mht[i,j] <- intercept[j] + ar1coef*mIt[imin1,j]
      mPIt[i,j] <- 1/(1 + exp(-mht[i,j]))
      mPIt[i,j] <- min(mPIt[i,j], 1-.Machine$double.eps) #ensure no exact 1s
      mPIt[i,j] <- max(mPIt[i,j], .Machine$double.eps) #ensure no exact 0s
    }

    ##simulate the It's:
    mIt[i,] <- rmbinary(1, probs=mPIt[i,], corrmat=cmat)
    
  } #close for(i){..}

  ##return result:
  ##==============
  
  mIt <- mIt[-c(1:burn.in),]
  return(mIt)
       
} #close dgpdSim() function
  

## this function computes the value used in the DF-adjustment of
## the DF-adjusted LM-statistic. Function argument:
##
##  S: number of periods
##
## value returned: a numeric, positive or negative

iDFlmFun <- function(S){
  sxfun <- function(S, location, speed){
    1/(1+exp( - speed * ( S - location ) ) )
  }
  result <- 60 +  20*sxfun(S, 8-4, 1) - 20*sxfun(S, 16-2, 0.5) -
    20*sxfun(S, 32-2, 0.5) - 20*sxfun(S, 48, 0.2) - 20*sxfun(S, 96-36, 0.2)
  return(result)
}


## This function computes the generalised inverse of a positive
## definite matrix. The code is originally Christian Francq's.
# Inverse (gnralise) d'une matrice symmetrique semi-definie positive:
gsolve <- function(Sigma, tol=sqrt(.Machine$double.eps)) {
  n<- nrow(Sigma)
  sp<-eigen(Sigma, TRUE)
  sp$vector%*%diag(1/pmax(abs(sp$values),tol))%*%t(sp$vector)
}


## This function computes the LM-statistic (HAC-version)
## without DF-adjustment. Arguments:
##
##  mIt:          The binary variates
##  mRt:          The restriction matrix
##  iL:           iL <- max(1,iS_T), where iS_T is the bandwidth          
##  vW:           ##Bartlett weights, computed as vW = 1 - 1:iL/iS_T (Hansen, 1992)
##
## value returned by function: a numeric value (the LM-statistic)

LMstat <- function(mIt, mRt=mRt, iS_T=iS_T)
{
  ##lag truncation:
  iS_T <- floor(iS_T)
  iL <- max(1,iS_T) #note: j/iS_T > 1 when j > i
  ##Bartlett weights:
  vW <- 1 - 1:iL/iS_T #Hansen (1992)
  #vW <- 1 - 1:iL/(iL+1) #Newey and West (1987, p. 704)
  
  ##rows and columns:
  iT <- NROW(mIt); S <- NCOL(mIt)
  
  ##unconstrained estimator:
  pi1hat <- colMeans(mIt)
  pi1hat <- pmin(pi1hat, 1-.Machine$double.eps) #ensure no exact 1s
  pi1hat <- pmax(pi1hat, .Machine$double.eps) #ensure no exact 0s

  ##constrained estimator:
  pi1tilde <- mean(pi1hat)
  pi1tilde <- rep(pi1tilde, S)

  ##build mAtilde:
  mAtilde <- matrix(0, S, S)
  for(j in 1:S){
    term1 <- (1-mIt[,j])/((1-pi1tilde[j])^2)
    term2 <- mIt[,j]/(pi1tilde[j]^2)
    mAtilde[j,j] <- mean(term1 + term2)
  }
  mAtildeInv <- solve(mAtilde)

  ##compute scores:
  mStilde <- matrix(NA, iT, S) #matrix of scores
  for(j in 1:S){
    term1 <- (1-mIt[,j])/(1-pi1tilde[j])
    term2 <- mIt[,j]/pi1tilde[j]
    mStilde[,j] <- term1 - term2
  }

  ##compute the "meat" (B-matrix) mBtilde:
  mGtilde0 <- crossprod(mStilde)/iT  #Gamma(j=0)
  mBtilde <- mGtilde0
  if( iL > 0 ){
    for(j in 1:iL){
      mS0 <- cbind(mStilde[-c(1:j),])
      mSj <- cbind(mStilde[-c(c(iT-j+1):iT),])
      mGtildej1 <- crossprod(mS0,mSj)/iT
      mGtildej2 <- crossprod(mSj,mS0)/iT
      mBtilde <- mBtilde + vW[j] * (mGtildej1 + mGtildej2)
    }
  }

  ##compute statistic, record p-value:
  mStildeSums <- matrix(NA, S, 1)
  mStildeSums[,1] <- colSums(mStilde)
  mStildeSumst <- t(mStildeSums)
  term1 <- (1/iT) * mStildeSumst %*% mAtildeInv %*% mRt
  if( S <= 2 ){
    term2 <- solve(mR %*% mAtildeInv %*% mBtilde %*% mAtildeInv %*% mRt)
  }else{
    term2 <- gsolve(mR %*% mAtildeInv %*% mBtilde %*% mAtildeInv %*% mRt)
  }
  term3 <- mR %*% mAtildeInv %*% mStildeSums
  Statistic <- term1 %*% term2 %*% term3

  ##return statistic:
  return(Statistic)

} #close LMstat()

                
###########################################################
## 3 SIMULATION PARAMETERS
###########################################################

iReps <- 1000 #no. of replications
alpha <- 0.05 #nominal significance level
DGP <- "A" #"A", "B", "C" or "D"
EIt <- 0.9 #non-zero probability
EIt_min <- 0.7 #minimum non-zero probability (for power simulations)
load_mIt <- FALSE #load (TRUE) or save (FALSE) the simulated I_st's
folder <- #where the simulations are loaded from/saved to
  "C:/Users/sucarrat/Documents/"
  
##which tests?
##============

doWaldTest <- FALSE #TRUE or FALSE
doLMTest <- TRUE #TRUE or FALSE
doLRTest <- FALSE #TRUE or FALSE

##covariance, DF-adjustment:
##==========================

HACvcov <- TRUE #TRUE or FALSE
DFadjust <- TRUE #TRUE or FALSE

##sample size T
##=============

##8-hour trading day: 60=quarter, 125=half-year, 250=year, 1000=4 years
vTvalues <- c(125, 250, 500, 1000)

##intraday period S 
##=================

SperiodsToConsider <- c(2,8,16,32,48,96)

##bandwidths
##==========

##vSTvalues contains the values of S_T function.
##note: length(vSTvalues) should be the same as length(vTvalues)
if( HACvcov==FALSE ){
  vSTvalues <- rep(0,length(vTvalues))
}else{
  vSTvalues <- 2*(vTvalues/100)^(2/9)
}
##check length(vSTvalues):
if( length(vSTvalues) != length(vTvalues) ){
  print("WARNING: 'length(vSTvalues) != length(vTvalues)'")
}


###########################################################
## 4 SIMULATE
###########################################################

##create results matrices:
lengthSperiods <- length(SperiodsToConsider)
mResultsWald <- matrix(NA, length(vTvalues), lengthSperiods)
rownames(mResultsWald) <- paste0("T=", vTvalues)
colnames(mResultsWald) <- paste0("S=",SperiodsToConsider)
mResultsLM <- mResultsWald
mResultsLR <- mResultsWald

##loop over sample sizes
##======================

for(m in 1:length(vTvalues)){

  ##current sample size:
  iT <- vTvalues[m]

  ##set vSvalues:
  vSvalues <- SperiodsToConsider

  ##current S_T (bandwidth) value:
  iS_T <- vSTvalues[m]

  ##print loop no.:
  cat("T = ", iT, ": \n", sep="")

  ##Bartlett kernel
  ##---------------

  if( HACvcov==FALSE ){
    iL <- 0 #assumes scores not autocorrelated
  }
  if( HACvcov==TRUE ){
    ##lag truncation:
    iS_T <- floor(iS_T)
    iL <- max(1,iS_T) #note: j/iS_T > 1 when j > i
    ##Bartlett weights:
    vW <- 1 - 1:iL/iS_T #Hansen (1992)
  }

  ##loop over S-values
  ##------------------
  
  for(l in 1:length(vSvalues)){
  
    ##current value of S:
    S <- vSvalues[l] 
      
    ##print loop no.:
    cat("S = ", S, ": \n", sep="")
  
    ##DF-correction
    ##-------------

    if( DFadjust==FALSE ){
      iDFwald <- 0 #Wald-test
      iDFlm <- 0 #LM-test
      iDFlr <- 0 #LR-test
    }else{
      if( HACvcov==FALSE ){
        ##Wald-correction:
        iDFwald <- (7/5)*(S + 16 - 2*log(S-1))
        ##LM-correction:
        iDFlm <- 12 - 0.5*log(S-1)^2
        ##LR-correction:
        iDFlr <- 0
      }
      if( HACvcov==TRUE ){
        ##Wald-correction:
        iDFwald <- (7/5)*(S + 16 - 2*log(S-1))
        ##LM-correction:
        iDFlm <- iDFlmFun(S)
        ##LR-correction:
        iDFlr <- 0
      }      
    } #close if(..)else(..)

    ##compute DF-adjustments (the value 1 means no correction):
    DFwaldAdj <- 1 - iDFwald/iT
    DFwaldAdj <- max(0.1, DFwaldAdj)
    DFlmAdj <- 1 - iDFlm/iT
    DFlmAdj <- max(0.1, DFlmAdj)
    DFlrAdj <- 1 - iDFlr/iT
    DFlrAdj <- max(0.1, DFlrAdj)
  
    ##correlation vector
    ##------------------
    
    ##correlations:
    if( DGP=="A" ){
      corr.vec <- numeric(0)
      for(i in c(S-1):1){ corr.vec <- c(corr.vec, rep(0,i)) }
    }
    
    ##non-zero contemporaneous correlations:
    if( DGP %in% c("B","D") ){
      corrs <- c(0.20, 0.15, 0.10, 0.05, 0.01)
      corr.vec <- numeric(0)
      for(i in c(S-1):1){
        tmp <- rep(0,i)
        tmp[1:min(i,length(corrs))] <- corrs[1:min(i,length(corrs))]
        corr.vec <- c(corr.vec, tmp)
      }
    }
        
    ##create vector of EIt's and intercept coefs:
    if( EIt==EIt_min ){
      nonzeroprob <- rep(EIt, S)
      interceptCoef <- rep(1.4, S)
    }else if( EIt==0.9 && EIt_min==0.8 ){
      byValue <- (0.9 - 0.8)/(S-1)
      nonzeroprob <- seq(EIt, EIt_min, by=-byValue)
      byValue <- (1.4 - 0.7)/(S-1)
      interceptCoef <- seq(1.4,0.7,by=-byValue)
    }else if( EIt==0.9 && EIt_min==0.7 ){
      byValue <- (0.9 - 0.7)/(S-1)
      nonzeroprob <- seq(EIt, EIt_min, by=-byValue)
      byValue <- (1.4 - 0.25)/(S-1)
      interceptCoef <- seq(1.4,0.25,by=-byValue)
    }

    ##sigma star and cmat for DGPs A, B and D:
    if( DGP %in% c("A","B","D") ){
      cmat <- lower.tri.to.corr.mat(corr.vec, S)
      ##for A and B:
      invisible(
        capture.output(
          sigma.star <- compute.sigma.star(no.bin=S, no.nor=0,
            prop.vec.bin=nonzeroprob, corr.mat=cmat)
        )
      )
    }
          
    ##build restriction matrix
    ##------------------------
    
    ##preferred version:
    tmp1 <- matrix(0, S-1, S-1)
    diag(tmp1) <- 1
    tmp2 <- matrix(0, S-1, S-1)
    diag(tmp2) <- -1
    tmp1 <- cbind(tmp1,0)
    tmp2 <- cbind(0,tmp2)
    mR <- tmp1 + tmp2
        
    ##transpose R:
    mRt <- t(mR)
  
    ##loop over replications
    ##----------------------
    
    ##results matrices:
    mWald <- matrix(NA, iReps, 2)
    colnames(mWald) <- c("Statistic", "pvalue")
    mLM <- matrix(NA, iReps, 2)
    colnames(mLM) <- c("Statistic", "pvalue")
    mLR <- matrix(NA, iReps, 2)
    colnames(mLR) <- c("Statistic", "pvalue")
    
    ##matrices for estimates:
    mPI1hat <- matrix(NA, iReps, S)
    mPI1tilde <- matrix(NA, iReps, S)
    
    ##list for simulations:
    if( load_mIt ){
      filename <- paste0(
        folder,
        paste0("dgp", DGP, "_EIt_", EIt, "to", EIt_min, "_S", S)
        )
      load(filename)
    }else{
      list_mIt <- as.list(rep(NA, iReps))
    }
    
    ##do the loop over replications: 
    for(i in 1:iReps){
    
      ##simulations started at:
      if(i==1){ cat("  Simulations started at: ", substr(Sys.time(), 1, 19),
          " \n", sep="") }
      
      ##simulate It:
      if( load_mIt ){
        mIt <- list_mIt[[i]]
        mIt <- mIt[1:iT,]        
      }else{

        if( DGP %in% c("A","B") ){
          invisible(
            capture.output(
              mIt <- jointly.generate.binary.normal(iT, S, no.nor=0,
                prop.vec.bin=nonzeroprob, sigma_star=sigma.star$sigma_star,
                continue.with.warning=FALSE)
            )
          )
        }else if( DGP =="C" ){
          mIt <- matrix(NA,iT,S)
          for(j in 1:S){
            mIt[,j] <- logitxSim(iT, intercept=interceptCoef[j], ar=0.9, as.zoo=FALSE)
          }
        }else if( DGP=="D" ){
          mIt <- dgpdSim(iT, burn.in=20, S=S, cmat=cmat,
            intercept = interceptCoef, ar1coef = 0.9)
        }
        colnames(mIt) <- paste0("s", 1:S)
        rownames(mIt) <- paste0("t = ", 1:iT)
        list_mIt[[i]] <- mIt
        
      }
        
      ##estimate pi1 = E(It)
      ##-------------------
      
      ##unconstrained estimator:
      pi1hat <- colMeans(mIt) 
      pi1hat <- pmin(pi1hat, 1-.Machine$double.eps) #ensure no exact 1s
      pi1hat <- pmax(pi1hat, .Machine$double.eps) #ensure no exact 0s
      mPI1hat[i,] <- pi1hat

      ##constrained estimator:
      pi1tilde <- mean(pi1hat)
      pi1tilde <- rep(pi1tilde, S)
      mPI1tilde[i,] <- pi1tilde
      
      ##Wald test
      ##---------
    
      if( doWaldTest ){
        ##build mAhat:
        mAhat <- matrix(0, S, S) 
        for(j in 1:S){
          term1 <- (1-mIt[,j])/((1-pi1hat[j])^2)
          term2 <- mIt[,j]/(pi1hat[j]^2)
          mAhat[j,j] <- mean(term1 + term2)
        }
        mAhatInv <- solve(mAhat)
          
        ##compute scores:
        mShat <- matrix(NA, iT, S) #matrix of scores
        for(j in 1:S){
          term1 <- (1-mIt[,j])/(1-pi1hat[j])
          term2 <- mIt[,j]/pi1hat[j]
          mShat[,j] <- term1 - term2
        }
        
        ##compute the "meat" (B-matrix) mBhat:
        mGhat0 <- crossprod(mShat)/iT  #Gamma(j=0)
        mBhat <- mGhat0
        if( iL > 0 ){
          for(j in 1:iL){
            mS0 <- cbind(mShat[-c(1:j),])
            mSj <- cbind(mShat[-c(c(iT-j+1):iT),])
            mGhatj1 <- crossprod(mS0,mSj)/iT
            mGhatj2 <- crossprod(mSj,mS0)/iT
            mBhat <- mBhat + vW[j] * (mGhatj1 + mGhatj2)
          }
        }
        
        ##compute vcov:
        mSigmahat <- mAhatInv %*% mBhat %*% mAhatInv
      
        ##compute statistic, record p-value:
        tmp <- mR %*% pi1hat
        Statistic <-
          iT * t(tmp) %*% solve(mR %*% mSigmahat %*% t(mR)) %*% tmp
        Statistic <- DFwaldAdj * Statistic ##DF-adjust
        pvalue <- pchisq(Statistic, S-1, lower.tail=FALSE) ##p-value
        mWald[i,"Statistic"] <- Statistic
        mWald[i,"pvalue"] <- pvalue
      } #close if( doWaldTest )
            
      ##Lagrange Multiplier test
      ##------------------------
            
      if( doLMTest ){
        ##build mAtilde:
        mAtilde <- matrix(0, S, S) 
        for(j in 1:S){
          term1 <- (1-mIt[,j])/((1-pi1tilde[j])^2)
          term2 <- mIt[,j]/(pi1tilde[j]^2)
          mAtilde[j,j] <- mean(term1 + term2)
        }
        mAtildeInv <- solve(mAtilde)
      
        ##compute scores:
        mStilde <- matrix(NA, iT, S) #matrix of scores
        for(j in 1:S){
          term1 <- (1-mIt[,j])/(1-pi1tilde[j])
          term2 <- mIt[,j]/pi1tilde[j]
          mStilde[,j] <- term1 - term2
        }
      
        ##compute the "meat" (B-matrix) mBtilde:
        mGtilde0 <- crossprod(mStilde)/iT  #Gamma(j=0)
        mBtilde <- mGtilde0
        if( iL > 0 ){
          for(j in 1:iL){
            mS0 <- cbind(mStilde[-c(1:j),])
            mSj <- cbind(mStilde[-c(c(iT-j+1):iT),])
            mGtildej1 <- crossprod(mS0,mSj)/iT
            mGtildej2 <- crossprod(mSj,mS0)/iT
            mBtilde <- mBtilde + vW[j] * (mGtildej1 + mGtildej2)
          }
        }
      
        ##compute statistic, record p-value:
        mStildeSums <- matrix(NA, S, 1)
        mStildeSums[,1] <- colSums(mStilde)
        mStildeSumst <- t(mStildeSums)  
        term1 <- (1/iT) * mStildeSumst %*% mAtildeInv %*% mRt
        term2 <- solve(mR %*% mAtildeInv %*% mBtilde %*% mAtildeInv %*% mRt)
        term3 <- mR %*% mAtildeInv %*% mStildeSums
        Statistic <- term1 %*% term2 %*% term3
        Statistic <- DFlmAdj * Statistic ##DF-adjust
        pvalue <- pchisq(Statistic, S-1, lower.tail=FALSE) ##p-value
        mLM[i,"Statistic"] <- Statistic
        mLM[i,"pvalue"] <- pvalue

      } #close if( doLMTest )

      ##Likelihood ratio test
      ##---------------------
      
      if( doLRTest ){
        ##log-likelihood hat:
        mLogLikHat <- matrix(NA, NROW(mIt), NCOL(mIt))
        for(j in 1:NCOL(mLogLikHat)){
          mLogLikHat[,j] <-
            mIt[,j] * ( log(1-pi1hat[j]) - log(pi1hat[j]) ) - log(1-pi1hat[j])
        }        
        LogLikHatSum <- sum( rowSums(mLogLikHat) )
              
        ##log-likelihood tilde:
        mLogLikTilde <- matrix(NA, NROW(mIt), NCOL(mIt))
        for(j in 1:NCOL(mLogLikTilde)){
          mLogLikTilde[,j] <-
            mIt[,j] * ( log(1-pi1tilde[j]) - log(pi1tilde[j]) ) - log(1-pi1tilde[j])
        }        
        LogLikTildeSum <- sum( rowSums(mLogLikTilde) )
        
        ##compute statistic, record p-value:
        Statistic <- 2 * (LogLikTildeSum - LogLikHatSum)
        Statistic <- DFlrAdj * Statistic ##DF-adjust
        pvalue <- pchisq(Statistic, S-1, lower.tail=FALSE) ##p-value
        mLR[i,"Statistic"] <- Statistic
        mLR[i,"pvalue"] <- pvalue
      } #close if( doLRTest )
      
      ##print loop no.
      ##--------------
      
      if( any(i/iReps == c(0.001,0.01,0.02,0.04,0.1,0.2,0.5,0.7,0.9,0.95,0.99)) ){
        cat("  Replication ", i, " at ", substr(Sys.time(), 1, 19),
          "\n", sep="")
      }

    } #close replication loop
  
  
    ##record results
    ##--------------
  
    ##wald test:
    if( doWaldTest ){ mResultsWald[m,l] <- mean( mWald[,"pvalue"] <= alpha ) }
    ##Lagrange Multiplier (LM) test:
    if( doLMTest ){ mResultsLM[m,l] <- mean( mLM[,"pvalue"] <= alpha ) }
    ##Likelihood Ratio (LR) test:
    if( doLRTest ){ mResultsLR[m,l] <- mean( mLR[,"pvalue"] <= alpha ) }
    
    ##store the mIt's:
    ##----------------
    
    if( load_mIt==FALSE ){
      filename <- paste0(
        folder,
        paste0("dgp", DGP, "_EIt_", EIt, "to", EIt_min, "_S", S)
        )
      save(list_mIt, file=filename)
    }
  
  } #close loop over S

  ##print results for current iT
  ##----------------------------
    
  if( doWaldTest ){ totex( rbind(mResultsWald[m,]) ) }
  if( doLMTest ){ totex( rbind(mResultsLM[m,]) ) }
  if( doLRTest ){ totex( rbind(mResultsLR[m,]) ) }

} #close loop over sample sizes (vTvalues)


##beep to notify simulations are finished
##=======================================

alarm()


###########################################################
##5 RESULTS
###########################################################

##print results
mResultsWald
mResultsLM
mResultsLR

##simple LaTeX code:
totex(mResultsWald)
totex(mResultsLM)
totex(mResultsLR)
